#!/usr/bin/python3
from PyQt5.QtCore import QProcess, QSysInfo, Qt, QEvent, QSettings, QPoint, QSize
from PyQt5.QtWidgets import QWidget, QApplication,  QPlainTextEdit, QVBoxLayout, QMainWindow, QAction
from PyQt5.QtGui import QIcon, QTextCursor
import sys
import getpass
import socket
import os
from PyQt5.QtCore import pyqtSignal
from function.cores.stdedit import stdinit

class Std_Termainal(QMainWindow):
    termin = pyqtSignal(str)  # 0 settext 1 appead 3process

    def __init__(self):
        super(Std_Termainal, self).__init__()
        try:
            # self.setStyleSheet(mystylesheet(self))

            self.commandslist = []
            self.tracker = 0
            # os.chdir(os.path.expanduser("~"))
            #        print(os.getcwd())
            # self.name = (str(getpass.getuser()) + "@" + str(socket.gethostname())+ ":" + str(os.getcwd()) + "$ ")
            self.name = "GDB>"
            self.setWindowTitle('Stdterminal')
            self.setWindowIcon(QIcon.fromTheme("terminal-emulator"))
            self.process = QProcess(self)
            self.process.setProcessChannelMode(QProcess.MergedChannels)
            self.process.readyRead.connect(self.dataReady)
            self.process.readyReadStandardError.connect(self.onReadyReadStandardError)
            self.process.readyReadStandardOutput.connect(self.onReadyReadStandardOutput)
            self.process.finished.connect(self.isFinished)
            self.process.setWorkingDirectory(os.getcwd())
            self.createStatusBar()

            self.commandfield = QPlainTextEdit()
            self.commandfield.setLineWrapMode(QPlainTextEdit.NoWrap)
            self.commandfield.setAttribute(Qt.WA_InputMethodEnabled,
                                           False)  # setAttribute(Qt::WA_InputMethodEnabled, false)
            self.commandfield.setFixedHeight(44)
            self.commandfield.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
            self.commandfield.setAcceptDrops(True)
            self.cursor = self.commandfield.textCursor()

            self.textWindow = QPlainTextEdit(self)

            self.textWindow.setReadOnly(True)

            layout = QVBoxLayout()
            layout.addWidget(self.textWindow)
            layout.addWidget(self.commandfield)
            self.wid = QWidget()
            self.wid.setLayout(layout)
            self.setCentralWidget(self.wid)
            self.setGeometry(0, 0, 600, 500)
            self.commandfield.setPlainText(self.name)

            self.commandfield.setFocus()

            self.copySelectedTextAction = QAction(QIcon.fromTheme("edit-copy"), "Copy", shortcut="Shift+Ctrl+c",
                                                  triggered=self.copyText)
            self.textWindow.addAction(self.copySelectedTextAction)
            self.pasteTextAction = QAction(QIcon.fromTheme("edit-paste"), "Copy", shortcut="Shift+Ctrl+v",
                                           triggered=self.pasteText)
            self.commandfield.addAction(self.pasteTextAction)
            #
            # self.cancelAction = QAction("Cancel", shortcut="Ctrl+c", triggered=self.killProcess)
            # self.textWindow.addAction(self.cancelAction)

            self.commandfield.installEventFilter(self)
            #        self.textWindow.installEventFilter(self)
            QApplication.setCursorFlashTime(1000)
            self.cursorEnd()
            # print(self.process.workingDirectory())
            self.settings = QSettings("QTerminal", "QTerminal")
            self.readSettings()
            self.commandfield.setCursorWidth(5)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def closeEvent(self, e):
        try:
            self.writeSettings()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def cursorEnd(self):
        try:
            # self.name = (str(getpass.getuser()) + "@" + str(socket.gethostname())+ ":" + str(os.getcwd()) + "$ ")
            self.name = "GDB>"
            self.commandfield.setPlainText(self.name)
            cursor = self.commandfield.textCursor()
            cursor.movePosition(11, 0)

            self.commandfield.setTextCursor(cursor)
            self.commandfield.setFocus()
        except:
            stdinit.std_signal_gobal.stdprintln()



    def eventFilter(self, source, event):
        try:
            if source == self.commandfield:
                if (event.type() == QEvent.DragEnter):
                    event.accept()
                    return True
                elif (event.type() == QEvent.Drop):
                    print('Drop')
                    self.setDropEvent(event)
                    return True
                elif (event.type() == QEvent.KeyPress):
                    cursor = self.commandfield.textCursor()
                    #                print('key press:', (event.key(), event.text()))
                    if event.key() == Qt.Key_Backspace:
                        if cursor.positionInBlock() <= len(self.name):
                            return True
                        else:
                            return False

                    elif event.key() == Qt.Key_Return:
                        self.run()
                        return True

                    elif event.key() == Qt.Key_Left:
                        if cursor.positionInBlock() <= len(self.name):
                            return True
                        else:
                            return False

                    elif event.key() == Qt.Key_Delete:
                        if cursor.positionInBlock() <= len(self.name) - 1:
                            return True
                        else:
                            return False

                    # elif event.modifiers() == Qt.ControlModifier and event.key() == Qt.Key_C:
                    #     self.killProcess()
                    #     return True

                    elif event.key() == Qt.Key_Up:
                        try:
                            if self.tracker != 0:
                                cursor.select(QTextCursor.BlockUnderCursor)
                                cursor.removeSelectedText()
                                self.commandfield.appendPlainText(self.name)

                            self.commandfield.insertPlainText(self.commandslist[self.tracker])
                            self.tracker -= 1

                        except IndexError:
                            self.tracker = 0
                        return True

                    elif event.key() == Qt.Key_Down:
                        try:
                            if self.tracker != 0:
                                cursor.select(QTextCursor.BlockUnderCursor)
                                cursor.removeSelectedText()
                                self.commandfield.appendPlainText(self.name)

                            self.commandfield.insertPlainText(self.commandslist[self.tracker])
                            self.tracker += 1

                        except IndexError:
                            self.tracker = 0
                        return True

                    else:
                        return False
                else:
                    return False
            else:
                return False
        except:
            stdinit.std_signal_gobal.stdprintln()




    def copyText(self):
        try:
            self.textWindow.copy()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def pasteText(self):
        try:
            self.commandfield.paste()
        except:
            stdinit.std_signal_gobal.stdprintln()


    # def killProcess(self):
    #     #print("cancelled")
    #     self.process.kill()
    #     #self.textWindow.appendPlainText("cancelled")
    #     self.cursorEnd()

    def createStatusBar(self):
        try:
            sysinfo = QSysInfo()
            # myMachine = "current CPU Architecture: " + sysinfo.currentCpuArchitecture() + " *** " + sysinfo.prettyProductName() + " *** " + sysinfo.kernelType() + " " + sysinfo.kernelVersion()
            myMachine = "GDB"
            self.statusBar().showMessage(myMachine, 0)
        except:
            stdinit.std_signal_gobal.stdprintln()



    def setDropEvent(self, event):
        try:
            self.commandfield.setFocus()
            if event.mimeData().hasUrls():
                f = str(event.mimeData().urls()[0].toLocalFile())
                print("is file:", f)
                if " " in f:
                    self.commandfield.insertPlainText("'{}'".format(f))
                else:
                    self.commandfield.insertPlainText(f)
                event.accept()
            elif event.mimeData().hasText():
                ft = event.mimeData().text()
                print("is text:", ft)
                if " " in ft:
                    self.commandfield.insertPlainText("'{}'".format(ft))
                else:
                    self.commandfield.insertPlainText(ft)
            else:
                event.ignore()
        except:
            stdinit.std_signal_gobal.stdprintln()

    def back_text(self,text):
        try:
            if "Couldn't determine a path for the index cache directory" in text:
                pass
                self.textWindow.setFocus()
                self.textWindow.setPlainText("Start debugging~")
                self.cursorEnd()

            else:
                self.textWindow.setFocus()
                self.textWindow.appendPlainText(str(text))
                self.cursorEnd()
        except:
            stdinit.std_signal_gobal.stdprintln()




    # Target is not responding, retrying...
    #
    # Debugger connection lost.
    #
    # Shutting down...
    def run(self):
        #print("started")

        try:
            cmd = self.commandfield.toPlainText()
            # t = ""
            self.textWindow.setFocus()
            self.textWindow.appendPlainText(cmd)
            # print(self.commandfield.toPlainText())
            cmd = cmd.replace(self.name, '')
            self.termin.emit(cmd)
            # print("command not found ...")
            # self.textWindow.appendPlainText("command not found ...")
            self.cursorEnd()
        except:
            stdinit.std_signal_gobal.stdprintln()

        #cli = []



        #cli = shlex.split(self.commandfield.toPlainText().replace(self.name, '').replace("'", '"'), posix=False)
        #cmd = str(cli)  ### is the executable
        #print(cmd)


        # if cmd == "exit":
        #     quit()
        #
        # elif cmd == "cd":
        #     del cli[0]
        #     path = " ".join(cli)
        #     os.chdir(os.path.abspath(path))
        #     self.process.setWorkingDirectory(os.getcwd())
        #     print("workingDirectory:", self.process.workingDirectory())
        #     self.cursorEnd()
        # else:
        #     self.process.setWorkingDirectory(os.getcwd())
        #     print("workingDirectory", self.process.workingDirectory())
        #     del cli[0]
        #     if (QStandardPaths.findExecutable(cmd)):
        #         self.commandslist.append(self.commandfield.toPlainText().replace(self.name, ""))
        #         print("command", cmd, "found")
        #         t = " ".join(cli)
        #         if self.process.state() != 2:
        #             self.process.waitForStarted()
        #             self.process.waitForFinished()
        #             if "|" in t or ">" in t or "<" in t:
        #                 print("special characters")
        #                 self.process.start('sh -c "' + cmd + ' ' + t + '"')
        #                 print("running", ('sh -c "' + cmd + ' ' + t + '"'))
        #             else:
        #                 self.process.start(cmd + " " + t)
        #                 print("running", (cmd + " " + t))
        #     else:
        #         print("command not found ...")
        #         self.textWindow.appendPlainText("command not found ...")
        #         self.cursorEnd()


    def dataReady(self):
        out = ""
        try:
            out = str(self.process.readAll(), encoding='utf8').rstrip()
        except TypeError:
            stdinit.std_signal_gobal.stdprintln()
            out = str(self.process.readAll()).rstrip()
            self.textWindow.moveCursor(self.cursor.Start)  ### changed
        self.textWindow.appendPlainText(out)

    def onReadyReadStandardError(self):
        try:
            self.error = self.process.readAllStandardError().data().decode()
            self.textWindow.appendPlainText(self.error.strip('\n'))
            self.cursorEnd()
        except TypeError:
            stdinit.std_signal_gobal.stdprintln()


    def onReadyReadStandardOutput(self):
        try:
            self.result = self.process.readAllStandardOutput().data().decode()
            self.textWindow.appendPlainText(self.result.strip('\n'))
            self.cursorEnd()
            self.state = self.process.state()
        except TypeError:
            stdinit.std_signal_gobal.stdprintln()


    def isFinished(self):
        try:

            self.name = (str(getpass.getuser()) + "@" + str(socket.gethostname())
                         + ":" + str(os.getcwd()) + "$ ")
            self.commandfield.setPlainText(self.name)
            self.cursorEnd()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def readSettings(self):
        try:
            if self.settings.contains("commands"):
                self.commandslist = self.settings.value("commands")
            if self.settings.contains("pos"):
                pos = self.settings.value("pos", QPoint(200, 200))
                self.move(pos)
            if self.settings.contains("size"):
                size = self.settings.value("size", QSize(400, 400))
                self.resize(size)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def writeSettings(self):
        try:
            self.settings.setValue("commands", self.commandslist)
            self.settings.setValue("pos", self.pos())
            self.settings.setValue("size", self.size())
        except:
            stdinit.std_signal_gobal.stdprintln()



def mystylesheet(self):
    return """
QMainWindow{
background-color: #212121; }

QMainWindow:title
{
background-color: QLinearGradient( x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #ffa02f, stop: 1 #ca0619);
color: #3465a4;
}

QPlainTextEdit
 {font-family: Noto Sans Mono; 
font-size: 8pt; 
background-color: #212121; 
color: #FFD700; padding: 2; 
border: none;}

QPlainTextEdit:focus { 
border: none; }

QScrollBar {            
border: 1px solid #2e3436;
background: #292929;
width:8px;
height: 8px;
margin: 0px 0px 0px 0px;
}
QScrollBar::handle {
background: #2e3436;
min-height: 0px;
min-width: 0px;
}
QScrollBar::add-line {
background: #2e3436;
height: 0px;
width: 0px;
subcontrol-position: bottom;
subcontrol-origin: margin;
}
QScrollBar::sub-line {
background: #2e3436;
height: 0px;
width: 0px;
subcontrol-position: top;
subcontrol-origin: margin;
}

QStatusBar {
font-family: Noto Sans Mono; 
font-size: 7pt; 
color: #729fcf;}

"""


def main():
    app = QApplication(sys.argv)
    ex = Std_Termainal()
    ex.show()
    #ex.raise_()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'demo_1.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore,  QtWidgets
from .stdedit import stdinit

class Std_Serial_Ui(object):
    def setupUi(self, Form):
        try:
            Form.setObjectName("Std_Serial_Ui")
            Form.resize(707, 658)  # 458
            self.formGroupBox = QtWidgets.QGroupBox(Form)  # x y Lenth with
            self.formGroupBox.setGeometry(QtCore.QRect(20, 280, 175, 240))
            self.formGroupBox.setObjectName("formGroupBox")
            self.formLayout = QtWidgets.QFormLayout(self.formGroupBox)
            self.formLayout.setContentsMargins(10, 10, 10, 10)  # 串口检测区
            self.formLayout.setSpacing(10)
            self.formLayout.setObjectName("formLayout")
            self.s1__lb_1 = QtWidgets.QLabel(self.formGroupBox)
            self.s1__lb_1.setObjectName("s1__lb_1")
            self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.s1__lb_1)
            self.s1__box_1 = QtWidgets.QPushButton(self.formGroupBox)
            self.s1__box_1.setAutoRepeatInterval(100)
            self.s1__box_1.setDefault(True)
            self.s1__box_1.setObjectName("s1__box_1")
            self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.s1__box_1)
            self.s1__lb_2 = QtWidgets.QLabel(self.formGroupBox)
            self.s1__lb_2.setObjectName("s1__lb_2")
            self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.s1__lb_2)
            self.s1__box_2 = QtWidgets.QComboBox(self.formGroupBox)
            self.s1__box_2.setObjectName("s1__box_2")
            self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.s1__box_2)
            self.s1__lb_3 = QtWidgets.QLabel(self.formGroupBox)
            self.s1__lb_3.setObjectName("s1__lb_3")
            self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.s1__lb_3)
            self.s1__box_3 = QtWidgets.QComboBox(self.formGroupBox)
            self.s1__box_3.setObjectName("s1__box_3")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.s1__box_3.addItem("")
            self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.s1__box_3)
            # self.s1__lb_4 = QtWidgets.QLabel(self.formGroupBox)
            # self.s1__lb_4.setObjectName("s1__lb_4")
            # self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.s1__lb_4)
            # self.s1__box_4 = QtWidgets.QComboBox(self.formGroupBox)
            # self.s1__box_4.setObjectName("s1__box_4")
            # self.s1__box_4.addItem("")
            # self.s1__box_4.addItem("")
            # self.s1__box_4.addItem("")
            # self.s1__box_4.addItem("")
            # self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.s1__box_4)
            # self.s1__lb_5 = QtWidgets.QLabel(self.formGroupBox)
            # self.s1__lb_5.setObjectName("s1__lb_5")
            # self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.s1__lb_5)
            # self.s1__box_5 = QtWidgets.QComboBox(self.formGroupBox)
            # self.s1__box_5.setObjectName("s1__box_5")
            # self.s1__box_5.addItem("")
            # self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.s1__box_5)
            self.open_button = QtWidgets.QPushButton(self.formGroupBox)
            self.open_button.setObjectName("open_button")
            self.formLayout.setWidget(7, QtWidgets.QFormLayout.SpanningRole, self.open_button)
            self.close_button = QtWidgets.QPushButton(self.formGroupBox)
            self.close_button.setObjectName("close_button")
            self.formLayout.setWidget(8, QtWidgets.QFormLayout.SpanningRole, self.close_button)
            # self.s1__lb_6 = QtWidgets.QLabel(self.formGroupBox)
            # self.s1__lb_6.setObjectName("s1__lb_6")
            # self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.s1__lb_6)
            # self.s1__box_6 = QtWidgets.QComboBox(self.formGroupBox)
            # self.s1__box_6.setObjectName("s1__box_6")
            # self.s1__box_6.addItem("")
            # self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.s1__box_6)
            self.state_label = QtWidgets.QLabel(self.formGroupBox)
            self.state_label.setText("")
            self.state_label.setTextFormat(QtCore.Qt.AutoText)
            self.state_label.setScaledContents(True)
            self.state_label.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignTrailing | QtCore.Qt.AlignVCenter)
            self.state_label.setObjectName("state_label")
            self.formLayout.setWidget(2, QtWidgets.QFormLayout.SpanningRole, self.state_label)
            self.verticalGroupBox = QtWidgets.QGroupBox(Form)
            self.verticalGroupBox.setGeometry(QtCore.QRect(10, 20, 701, 241))  # 接收区
            self.verticalGroupBox.setObjectName("verticalGroupBox")
            self.verticalLayout = QtWidgets.QVBoxLayout(self.verticalGroupBox)
            self.verticalLayout.setContentsMargins(10, 10, 10, 10)
            self.verticalLayout.setObjectName("verticalLayout")
            self.s2__receive_text = QtWidgets.QTextBrowser(
                self.verticalGroupBox)  # QtWidgets.QTextEdit(self.verticalGroupBox)#
            # self.s2__receive_text.setReadOnly(True)
            self.s2__receive_text.setObjectName("s2__receive_text")
            self.verticalLayout.addWidget(self.s2__receive_text)
            self.verticalGroupBox_2 = QtWidgets.QGroupBox(Form)
            self.verticalGroupBox_2.setGeometry(QtCore.QRect(210, 280, 500, 101))
            self.verticalGroupBox_2.setObjectName("verticalGroupBox_2")
            self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.verticalGroupBox_2)
            self.verticalLayout_2.setContentsMargins(10, 10, 10, 10)  # 发送区
            self.verticalLayout_2.setObjectName("verticalLayout_2")
            self.s3__send_text = QtWidgets.QTextEdit(self.verticalGroupBox_2)
            self.s3__send_text.setObjectName("s3__send_text")
            self.verticalLayout_2.addWidget(self.s3__send_text)
            self.s3__send_button = QtWidgets.QPushButton(Form)
            self.s3__send_button.setGeometry(QtCore.QRect(635, 400, 70, 30))  # 发送按钮
            self.s3__send_button.setObjectName("s3__send_button")
            # self.s3__clear_button = QtWidgets.QPushButton(Form)
            # self.s3__clear_button.setGeometry(QtCore.QRect(620, 350, 61, 31))#清除按钮
            # self.s3__clear_button.setObjectName("s3__clear_button")
            self.formGroupBox1 = QtWidgets.QGroupBox(Form)
            self.formGroupBox1.setGeometry(QtCore.QRect(210, 415, 155, 105))  # 串口状态
            self.formGroupBox1.setObjectName("formGroupBox1")
            self.formLayout_2 = QtWidgets.QFormLayout(self.formGroupBox1)
            self.formLayout_2.setContentsMargins(10, 10, 10, 10)
            self.formLayout_2.setSpacing(10)
            self.formLayout_2.setObjectName("formLayout_2")
            self.label = QtWidgets.QLabel(self.formGroupBox1)
            self.label.setObjectName("label")
            self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
            self.label_2 = QtWidgets.QLabel(self.formGroupBox1)
            self.label_2.setObjectName("label_2")
            self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
            self.lineEdit = QtWidgets.QLineEdit(self.formGroupBox1)

            self.lineEdit.setObjectName("lineEdit")
            self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineEdit)
            self.lineEdit_2 = QtWidgets.QLineEdit(self.formGroupBox1)
            self.lineEdit_2.setObjectName("lineEdit_2")
            # self.lineEdit_2.setReadOnly(True)
            self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.lineEdit_2)
            self.hex_send = QtWidgets.QCheckBox(Form)
            self.hex_send.setGeometry(QtCore.QRect(415, 276, 85, 22))  # 串口发送
            self.hex_send.setObjectName("hex_send")
            self.hex_receive = QtWidgets.QCheckBox(Form)
            self.hex_receive.setGeometry(QtCore.QRect(310, 276, 85, 22))  # 串口接收
            self.hex_receive.setObjectName("hex_receive")
            self.time_dot = QtWidgets.QCheckBox(Form)
            self.time_dot.setGeometry(QtCore.QRect(530, 276, 85, 22))  # 串口接收
            self.time_dot.setObjectName("time_dot")
            self.s2__clear_button = QtWidgets.QPushButton(Form)
            self.s2__clear_button.setGeometry(QtCore.QRect(635, 263, 70, 30))  # 串口清除
            self.s2__clear_button.setObjectName("s2__clear_button")
            self.timer_send_cb = QtWidgets.QCheckBox(Form)
            self.timer_send_cb.setGeometry(QtCore.QRect(385, 400, 85, 22))  # 定时发送
            self.timer_send_cb.setObjectName("timer_send_cb")

            self.DTR = QtWidgets.QCheckBox(Form)
            self.DTR.setGeometry(QtCore.QRect(415, 450, 71, 22))  # DTR
            self.DTR.setObjectName("DTR")
            self.RTS = QtWidgets.QCheckBox(Form)
            self.RTS.setGeometry(QtCore.QRect(415, 480, 71, 22))  # RTS
            self.RTS.setObjectName("RTS")

            self.s4__calc = QtWidgets.QPushButton(Form)
            self.s4__calc.setGeometry(QtCore.QRect(635, 485, 80, 30))  # 计算器
            self.s4__calc.setObjectName("s4__calc")

            self.lineEdit_3 = QtWidgets.QLineEdit(Form)
            self.lineEdit_3.setGeometry(QtCore.QRect(475, 400, 61, 20))  # 发送框定时发送
            self.lineEdit_3.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignTrailing | QtCore.Qt.AlignVCenter)
            self.lineEdit_3.setObjectName("lineEdit_3")
            self.dw = QtWidgets.QLabel(Form)
            self.dw.setGeometry(QtCore.QRect(541, 400, 54, 20))
            self.dw.setObjectName("dw")
            self.verticalGroupBox.raise_()
            self.verticalGroupBox_2.raise_()
            self.formGroupBox.raise_()
            self.s3__send_button.raise_()
            # self.s3__clear_button.raise_()
            self.formGroupBox.raise_()
            self.hex_send.raise_()
            self.hex_receive.raise_()
            self.time_dot.raise_()
            self.s2__clear_button.raise_()
            self.timer_send_cb.raise_()
            self.DTR.raise_()
            self.RTS.raise_()
            self.s4__calc.raise_()

            self.lineEdit_3.raise_()
            self.dw.raise_()

            self.retranslateUi(Form)
            QtCore.QMetaObject.connectSlotsByName(Form)
        except:
            stdinit.std_signal_gobal.stdprintln()



    def retranslateUi(self, Form):
        try:
            _translate = QtCore.QCoreApplication.translate
            Form.setWindowTitle(_translate("Form", "Form"))
            self.formGroupBox.setTitle(_translate("Form", "串口设置"))
            self.s1__lb_1.setText(_translate("Form", "串口检测："))
            self.s1__box_1.setText(_translate("Form", "检测串口"))
            self.s1__lb_2.setText(_translate("Form", "串口选择："))
            self.s1__lb_3.setText(_translate("Form", "波特率："))
            self.s1__box_3.setItemText(0, _translate("Form", "9600"))
            self.s1__box_3.setItemText(1, _translate("Form", "2400"))
            self.s1__box_3.setItemText(2, _translate("Form", "4800"))
            self.s1__box_3.setItemText(3, _translate("Form", "14400"))
            self.s1__box_3.setItemText(4, _translate("Form", "19200"))
            self.s1__box_3.setItemText(5, _translate("Form", "38400"))
            self.s1__box_3.setItemText(6, _translate("Form", "57600"))
            self.s1__box_3.setItemText(7, _translate("Form", "115200"))
            self.s1__box_3.setItemText(8, _translate("Form", "76800"))
            self.s1__box_3.setItemText(9, _translate("Form", "12800"))
            self.s1__box_3.setItemText(10, _translate("Form", "230400"))
            self.s1__box_3.setItemText(11, _translate("Form", "460800"))
            # self.s1__lb_4.setText(_translate("Form", "数据位："))
            # self.s1__box_4.setItemText(0, _translate("Form", "8"))
            # self.s1__box_4.setItemText(1, _translate("Form", "7"))
            # self.s1__box_4.setItemText(2, _translate("Form", "6"))
            # self.s1__box_4.setItemText(python3, _translate("Form", "5"))
            # self.s1__lb_5.setText(_translate("Form", "校验位："))
            # self.s1__box_5.setItemText(0, _translate("Form", "N"))
            # self.s1__lb_6.setText(_translate("Form", "停止位："))
            # self.s1__box_6.setItemText(0, _translate("Form", "1"))

            self.open_button.setText(_translate("Form", "打开串口"))
            self.close_button.setText(_translate("Form", "关闭串口"))
            self.verticalGroupBox.setTitle(_translate("Form", "接收区"))
            self.verticalGroupBox_2.setTitle(_translate("Form", "发送区"))
            self.s3__send_text.setHtml(_translate("Form",
                                                  "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
                                                  "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
                                                  "p, li { white-space: pre-wrap; }\n"
                                                  "</style></head><body style=\" font-family:\'SimSun\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">123456</p></body></html>"))
            self.s3__send_button.setText(_translate("Form", "串口发送"))
            # self.s3__clear_button.setText(_translate("Form", "1清除"))
            self.formGroupBox1.setTitle(_translate("Form", "串口状态"))
            self.label.setText(_translate("Form", "已接收："))
            self.label_2.setText(_translate("Form", "已发送："))
            self.hex_send.setText(_translate("Form", "Hex发送"))
            self.hex_receive.setText(_translate("Form", "Hex接收"))
            self.time_dot.setText(_translate("Form", "时间戳"))
            self.s2__clear_button.setText(_translate("Form", "清除窗口"))
            self.timer_send_cb.setText(_translate("Form", "定时发送"))
            self.DTR.setText(_translate("Form", "DTR"))
            self.RTS.setText(_translate("Form", "RTS"))
            if stdinit.platform_is == "Win":
                self.s4__calc.setText(_translate("Form", "打开计算器"))
            self.lineEdit_3.setText(_translate("Form", "1000"))
            self.dw.setText(_translate("Form", "ms/次"))
        except:
            stdinit.std_signal_gobal.stdprintln()





# Copyright (c) 2014-present PlatformIO <contact@platformio.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=no-self-use, unused-argument, too-many-lines
# pylint: disable=too-many-instance-attributes, too-many-public-methods
# pylint: disable=assignment-from-no-return

from __future__ import absolute_import

import hashlib
import io
import os
import re
import sys

import click
import SCons.Scanner  # pylint: disable=import-error
from SCons.Script import ARGUMENTS  # pylint: disable=import-error
from SCons.Script import COMMAND_LINE_TARGETS  # pylint: disable=import-error
from SCons.Script import DefaultEnvironment  # pylint: disable=import-error

from platformio import exception, fs, util
from platformio.builder.tools import platformio as piotool
from platformio.clients.http import InternetIsOffline
from platformio.compat import IS_WINDOWS, hashlib_encode_data, string_types
from platformio.package.exception import UnknownPackageError
from platformio.package.manager.library import LibraryPackageManager
from platformio.package.manifest.parser import (
    ManifestParserError,
    ManifestParserFactory,
)
from platformio.package.meta import PackageItem
from platformio.project.options import ProjectOptions


class LibBuilderFactory(object):
    @staticmethod
    def new(env, path, verbose=int(ARGUMENTS.get("PIOVERBOSE", 0))):
        clsname = "UnknownLibBuilder"
        if os.path.isfile(os.path.join(path, "library.json")):
            clsname = "PlatformIOLibBuilder"
        else:
            used_frameworks = LibBuilderFactory.get_used_frameworks(env, path)
            common_frameworks = set(env.get("PIOFRAMEWORK", [])) & set(used_frameworks)
            if common_frameworks:
                clsname = "%sLibBuilder" % list(common_frameworks)[0].title()
            elif used_frameworks:
                clsname = "%sLibBuilder" % used_frameworks[0].title()

        obj = getattr(sys.modules[__name__], clsname)(env, path, verbose=verbose)

        # Handle PlatformIOLibBuilder.manifest.build.builder
        # pylint: disable=protected-access
        if isinstance(obj, PlatformIOLibBuilder) and obj._manifest.get("build", {}).get(
            "builder"
        ):
            obj = getattr(
                sys.modules[__name__], obj._manifest.get("build", {}).get("builder")
            )(env, path, verbose=verbose)

        assert isinstance(obj, LibBuilderBase)
        return obj

    @staticmethod
    def get_used_frameworks(env, path):
        if any(
            os.path.isfile(os.path.join(path, fname))
            for fname in ("library.properties", "keywords.txt")
        ):
            return ["arduino"]

        if os.path.isfile(os.path.join(path, "module.json")):
            return ["mbed"]

        include_re = re.compile(
            r'^#include\s+(<|")(Arduino|mbed)\.h(<|")', flags=re.MULTILINE
        )

        # check source files
        for root, _, files in os.walk(path, followlinks=True):
            if "mbed_lib.json" in files:
                return ["mbed"]
            for fname in files:
                if not fs.path_endswith_ext(
                    fname, piotool.SRC_BUILD_EXT + piotool.SRC_HEADER_EXT
                ):
                    continue
                with io.open(
                    os.path.join(root, fname), encoding="utf8", errors="ignore"
                ) as fp:
                    content = fp.read()
                if not content:
                    continue
                if "Arduino.h" in content and include_re.search(content):
                    return ["arduino"]
                if "mbed.h" in content and include_re.search(content):
                    return ["mbed"]
        return []


class LibBuilderBase(object):

    CLASSIC_SCANNER = SCons.Scanner.C.CScanner()
    CCONDITIONAL_SCANNER = SCons.Scanner.C.CConditionalScanner()
    # Max depth of nested includes:
    # -1 = unlimited
    # 0 - disabled nesting
    # >0 - number of allowed nested includes
    CCONDITIONAL_SCANNER_DEPTH = 99
    PARSE_SRC_BY_H_NAME = True

    _INCLUDE_DIRS_CACHE = None

    def __init__(self, env, path, manifest=None, verbose=False):
        self.env = env.Clone()
        self.envorigin = env.Clone()
        self.path = os.path.realpath(env.subst(path))
        self.verbose = verbose

        try:
            self._manifest = manifest if manifest else self.load_manifest()
        except ManifestParserError:
            click.secho(
                "Warning! Ignoring broken library manifest in " + self.path, fg="yellow"
            )
            self._manifest = {}

        self._is_dependent = False
        self._is_built = False
        self._depbuilders = []
        self._circular_deps = []
        self._processed_files = []

        # reset source filter, could be overridden with extra script
        self.env["SRC_FILTER"] = ""

        # process extra options and append to build environment
        self.process_extra_options()

    def __repr__(self):
        return "%s(%r)" % (self.__class__, self.path)

    def __contains__(self, path):
        p1 = self.path
        p2 = path
        if IS_WINDOWS:
            p1 = p1.lower()
            p2 = p2.lower()
        if p1 == p2:
            return True
        return os.path.commonprefix((p1 + os.path.sep, p2)) == p1 + os.path.sep

    @property
    def name(self):
        return self._manifest.get("name", os.path.basename(self.path))

    @property
    def version(self):
        return self._manifest.get("version")

    @property
    def dependencies(self):
        return self._manifest.get("dependencies")

    @property
    def src_filter(self):
        return piotool.SRC_FILTER_DEFAULT + [
            "-<example%s>" % os.sep,
            "-<examples%s>" % os.sep,
            "-<test%s>" % os.sep,
            "-<tests%s>" % os.sep,
        ]

    @property
    def include_dir(self):
        for name in ("include", "Include"):
            d = os.path.join(self.path, name)
            if os.path.isdir(d):
                return d
        return None

    @property
    def src_dir(self):
        for name in ("src", "Src"):
            d = os.path.join(self.path, name)
            if os.path.isdir(d):
                return d
        return self.path

    def get_include_dirs(self):
        items = []
        include_dir = self.include_dir
        if include_dir:
            items.append(include_dir)
        items.append(self.src_dir)
        return items

    @property
    def build_dir(self):
        lib_hash = hashlib.sha1(hashlib_encode_data(self.path)).hexdigest()[:3]
        return os.path.join(
            "$BUILD_DIR", "lib%s" % lib_hash, os.path.basename(self.path)
        )

    @property
    def build_flags(self):
        return None

    @property
    def build_unflags(self):
        return None

    @property
    def extra_script(self):
        return None

    @property
    def depbuilders(self):
        return self._depbuilders

    @property
    def dependent(self):
        return self._is_dependent

    @property
    def is_built(self):
        return self._is_built

    @property
    def lib_archive(self):
        return self.env.GetProjectOption("lib_archive")

    @property
    def lib_ldf_mode(self):
        return self.env.GetProjectOption("lib_ldf_mode")

    @staticmethod
    def validate_ldf_mode(mode):
        ldf_modes = ProjectOptions["env.lib_ldf_mode"].type.choices
        if isinstance(mode, string_types):
            mode = mode.strip().lower()
        if mode in ldf_modes:
            return mode
        try:
            return ldf_modes[int(mode)]
        except (IndexError, ValueError):
            pass
        return ProjectOptions["env.lib_ldf_mode"].default

    @property
    def lib_compat_mode(self):
        return self.env.GetProjectOption("lib_compat_mode")

    @staticmethod
    def validate_compat_mode(mode):
        compat_modes = ProjectOptions["env.lib_compat_mode"].type.choices
        if isinstance(mode, string_types):
            mode = mode.strip().lower()
        if mode in compat_modes:
            return mode
        try:
            return compat_modes[int(mode)]
        except (IndexError, ValueError):
            pass
        return ProjectOptions["env.lib_compat_mode"].default

    def is_platforms_compatible(self, platforms):
        return True

    def is_frameworks_compatible(self, frameworks):
        return True

    def load_manifest(self):
        return {}

    def process_extra_options(self):
        with fs.cd(self.path):
            self.env.ProcessFlags(self.build_flags)
            if self.extra_script:
                self.env.SConscriptChdir(1)
                self.env.SConscript(
                    os.path.realpath(self.extra_script),
                    exports={"env": self.env, "pio_lib_builder": self},
                )
            self.env.ProcessUnFlags(self.build_unflags)

    def process_dependencies(self):
        if not self.dependencies:
            return
        for item in self.dependencies:
            found = False
            for lb in self.env.GetLibBuilders():
                if item["name"] != lb.name:
                    continue
                found = True
                if lb not in self.depbuilders:
                    self.depend_recursive(lb)
                break

            if not found and self.verbose:
                sys.stderr.write(
                    "Warning: Ignored `%s` dependency for `%s` "
                    "library\n" % (item["name"], self.name)
                )

    def get_search_files(self):
        items = [
            os.path.join(self.src_dir, item)
            for item in self.env.MatchSourceFiles(self.src_dir, self.src_filter)
        ]
        include_dir = self.include_dir
        if include_dir:
            items.extend(
                [
                    os.path.join(include_dir, item)
                    for item in self.env.MatchSourceFiles(include_dir)
                ]
            )
        return items

    def _get_found_includes(  # pylint: disable=too-many-branches
        self, search_files=None
    ):
        # all include directories
        if not LibBuilderBase._INCLUDE_DIRS_CACHE:
            LibBuilderBase._INCLUDE_DIRS_CACHE = [
                self.env.Dir(d)
                for d in ProjectAsLibBuilder(
                    self.envorigin, "$PROJECT_DIR"
                ).get_include_dirs()
            ]
            for lb in self.env.GetLibBuilders():
                LibBuilderBase._INCLUDE_DIRS_CACHE.extend(
                    [self.env.Dir(d) for d in lb.get_include_dirs()]
                )

        # append self include directories
        include_dirs = [self.env.Dir(d) for d in self.get_include_dirs()]
        include_dirs.extend(LibBuilderBase._INCLUDE_DIRS_CACHE)

        result = []
        for path in search_files or []:
            if path in self._processed_files:
                continue
            self._processed_files.append(path)

            try:
                assert "+" in self.lib_ldf_mode
                candidates = LibBuilderBase.CCONDITIONAL_SCANNER(
                    self.env.File(path),
                    self.env,
                    tuple(include_dirs),
                    depth=self.CCONDITIONAL_SCANNER_DEPTH,
                )
                # mark candidates already processed via Conditional Scanner
                self._processed_files.extend(
                    [
                        c.get_abspath()
                        for c in candidates
                        if c.get_abspath() not in self._processed_files
                    ]
                )
            except Exception as e:  # pylint: disable=broad-except
                if self.verbose and "+" in self.lib_ldf_mode:
                    sys.stderr.write(
                        "Warning! Classic Pre Processor is used for `%s`, "
                        "advanced has failed with `%s`\n" % (path, e)
                    )
                candidates = LibBuilderBase.CLASSIC_SCANNER(
                    self.env.File(path), self.env, tuple(include_dirs)
                )

            # print(path, [c.get_abspath() for c in candidates])
            for item in candidates:
                if item not in result:
                    result.append(item)
                if not self.PARSE_SRC_BY_H_NAME:
                    continue
                _h_path = item.get_abspath()
                if not fs.path_endswith_ext(_h_path, piotool.SRC_HEADER_EXT):
                    continue
                _f_part = _h_path[: _h_path.rindex(".")]
                for ext in piotool.SRC_C_EXT + piotool.SRC_CXX_EXT:
                    if not os.path.isfile("%s.%s" % (_f_part, ext)):
                        continue
                    _c_path = self.env.File("%s.%s" % (_f_part, ext))
                    if _c_path not in result:
                        result.append(_c_path)

        return result

    def depend_recursive(self, lb, search_files=None):
        def _already_depends(_lb):
            if self in _lb.depbuilders:
                return True
            for __lb in _lb.depbuilders:
                if _already_depends(__lb):
                    return True
            return False

        # assert isinstance(lb, LibBuilderBase)
        if self != lb:
            if _already_depends(lb):
                if self.verbose:
                    sys.stderr.write(
                        "Warning! Circular dependencies detected "
                        "between `%s` and `%s`\n" % (self.path, lb.path)
                    )
                self._circular_deps.append(lb)
            elif lb not in self._depbuilders:
                self._depbuilders.append(lb)
                LibBuilderBase._INCLUDE_DIRS_CACHE = None
        lb.search_deps_recursive(search_files)

    def search_deps_recursive(self, search_files=None):
        if not self._is_dependent:
            self._is_dependent = True
            self.process_dependencies()

            if self.lib_ldf_mode.startswith("deep"):
                search_files = self.get_search_files()

        # when LDF is disabled
        if self.lib_ldf_mode == "off":
            return

        lib_inc_map = {}
        for inc in self._get_found_includes(search_files):
            for lb in self.env.GetLibBuilders():
                if inc.get_abspath() in lb:
                    if lb not in lib_inc_map:
                        lib_inc_map[lb] = []
                    lib_inc_map[lb].append(inc.get_abspath())
                    break

        for lb, lb_search_files in lib_inc_map.items():
            self.depend_recursive(lb, lb_search_files)

    def build(self):
        libs = []
        for lb in self._depbuilders:
            libs.extend(lb.build())
            # copy shared information to self env
            for key in ("CPPPATH", "LIBPATH", "LIBS", "LINKFLAGS"):
                self.env.PrependUnique(**{key: lb.env.get(key)})

        for lb in self._circular_deps:
            self.env.PrependUnique(CPPPATH=lb.get_include_dirs())

        if self._is_built:
            return libs
        self._is_built = True

        self.env.PrependUnique(CPPPATH=self.get_include_dirs())

        if self.lib_ldf_mode == "off":
            for lb in self.env.GetLibBuilders():
                if self == lb or not lb.is_built:
                    continue
                for key in ("CPPPATH", "LIBPATH", "LIBS", "LINKFLAGS"):
                    self.env.PrependUnique(**{key: lb.env.get(key)})

        do_not_archive = not self.lib_archive
        if not do_not_archive:
            nodes = self.env.CollectBuildFiles(
                self.build_dir, self.src_dir, self.src_filter
            )
            if nodes:
                libs.append(
                    self.env.BuildLibrary(
                        self.build_dir, self.src_dir, self.src_filter, nodes
                    )
                )
            else:
                do_not_archive = True
        if do_not_archive:
            self.env.BuildSources(self.build_dir, self.src_dir, self.src_filter)

        return libs


class UnknownLibBuilder(LibBuilderBase):
    pass


class ArduinoLibBuilder(LibBuilderBase):
    def load_manifest(self):
        manifest_path = os.path.join(self.path, "library.properties")
        if not os.path.isfile(manifest_path):
            return {}
        return ManifestParserFactory.new_from_file(manifest_path).as_dict()

    @property
    def include_dir(self):
        if not all(
            os.path.isdir(os.path.join(self.path, d)) for d in ("include", "src")
        ):
            return None
        return os.path.join(self.path, "include")

    def get_include_dirs(self):
        include_dirs = LibBuilderBase.get_include_dirs(self)
        if os.path.isdir(os.path.join(self.path, "src")):
            return include_dirs
        if os.path.isdir(os.path.join(self.path, "utility")):
            include_dirs.append(os.path.join(self.path, "utility"))
        return include_dirs

    @property
    def src_filter(self):
        src_dir = os.path.join(self.path, "src")
        if os.path.isdir(src_dir):
            # pylint: disable=no-member
            src_filter = LibBuilderBase.src_filter.fget(self)
            for root, _, files in os.walk(src_dir, followlinks=True):
                found = False
                for fname in files:
                    if fname.lower().endswith("asm"):
                        found = True
                        break
                if not found:
                    continue
                rel_path = root.replace(src_dir, "")
                if rel_path.startswith(os.path.sep):
                    rel_path = rel_path[1:] + os.path.sep
                src_filter.append("-<%s*.[aA][sS][mM]>" % rel_path)
            return src_filter

        src_filter = []
        is_utility = os.path.isdir(os.path.join(self.path, "utility"))
        for ext in piotool.SRC_BUILD_EXT + piotool.SRC_HEADER_EXT:
            # arduino ide ignores files with .asm or .ASM extensions
            if ext.lower() == "asm":
                continue
            src_filter.append("+<*.%s>" % ext)
            if is_utility:
                src_filter.append("+<utility%s*.%s>" % (os.path.sep, ext))
        return src_filter

    @property
    def dependencies(self):
        # do not include automatically all libraries for build
        # chain+ will decide later
        return None

    @property
    def lib_ldf_mode(self):
        # pylint: disable=no-member
        if not self._manifest.get("dependencies"):
            return LibBuilderBase.lib_ldf_mode.fget(self)
        missing = object()
        global_value = self.env.GetProjectConfig().getraw(
            "env:" + self.env["PIOENV"], "lib_ldf_mode", missing
        )
        if global_value != missing:
            return LibBuilderBase.lib_ldf_mode.fget(self)
        # automatically enable C++ Preprocessing in runtime
        # (Arduino IDE has this behavior)
        return "chain+"

    def is_frameworks_compatible(self, frameworks):
        return util.items_in_list(frameworks, ["arduino", "energia"])

    def is_platforms_compatible(self, platforms):
        return util.items_in_list(platforms, self._manifest.get("platforms") or ["*"])

    @property
    def build_flags(self):
        ldflags = [
            LibBuilderBase.build_flags.fget(self),  # pylint: disable=no-member
            self._manifest.get("ldflags"),
        ]
        if self._manifest.get("precompiled") in ("true", "full"):
            # add to LDPATH {build.mcu} folder
            board_config = self.env.BoardConfig()
            for key in ("build.mcu", "build.cpu"):
                libpath = os.path.join(self.src_dir, board_config.get(key, ""))
                if not os.path.isdir(libpath):
                    continue
                self.env.PrependUnique(LIBPATH=libpath)
                break
        ldflags = [flag for flag in ldflags if flag]  # remove empty
        return " ".join(ldflags) if ldflags else None


class MbedLibBuilder(LibBuilderBase):
    def load_manifest(self):
        manifest_path = os.path.join(self.path, "module.json")
        if not os.path.isfile(manifest_path):
            return {}
        return ManifestParserFactory.new_from_file(manifest_path).as_dict()

    @property
    def src_dir(self):
        if os.path.isdir(os.path.join(self.path, "source")):
            return os.path.join(self.path, "source")
        return LibBuilderBase.src_dir.fget(self)  # pylint: disable=no-member

    def get_include_dirs(self):
        include_dirs = LibBuilderBase.get_include_dirs(self)
        if self.path not in include_dirs:
            include_dirs.append(self.path)

        # library with module.json
        for p in self._manifest.get("extraIncludes", []):
            include_dirs.append(os.path.join(self.path, p))

        # old mbed library without manifest, add to CPPPATH all folders
        if not self._manifest:
            for root, _, __ in os.walk(self.path):
                part = root.replace(self.path, "").lower()
                if any(s in part for s in ("%s." % os.path.sep, "test", "example")):
                    continue
                if root not in include_dirs:
                    include_dirs.append(root)

        return include_dirs

    def is_frameworks_compatible(self, frameworks):
        return util.items_in_list(frameworks, ["mbed"])

    def process_extra_options(self):
        self._process_mbed_lib_confs()
        return super(MbedLibBuilder, self).process_extra_options()

    def _process_mbed_lib_confs(self):
        mbed_lib_paths = [
            os.path.join(root, "mbed_lib.json")
            for root, _, files in os.walk(self.path)
            if "mbed_lib.json" in files
        ]
        if not mbed_lib_paths:
            return None

        mbed_config_path = None
        for p in self.env.get("CPPPATH"):
            mbed_config_path = os.path.join(self.env.subst(p), "mbed_config.h")
            if os.path.isfile(mbed_config_path):
                break
            mbed_config_path = None
        if not mbed_config_path:
            return None

        macros = {}
        for mbed_lib_path in mbed_lib_paths:
            macros.update(self._mbed_lib_conf_parse_macros(mbed_lib_path))

        self._mbed_conf_append_macros(mbed_config_path, macros)
        return True

    @staticmethod
    def _mbed_normalize_macro(macro):
        name = macro
        value = None
        if "=" in macro:
            name, value = macro.split("=", 1)
        return dict(name=name, value=value)

    def _mbed_lib_conf_parse_macros(self, mbed_lib_path):
        macros = {}
        cppdefines = str(self.env.Flatten(self.env.subst("$CPPDEFINES")))
        manifest = fs.load_json(mbed_lib_path)

        # default macros
        for macro in manifest.get("macros", []):
            macro = self._mbed_normalize_macro(macro)
            macros[macro["name"]] = macro

        # configuration items
        for key, options in manifest.get("config", {}).items():
            if "value" not in options:
                continue
            macros[key] = dict(
                name=options.get("macro_name"), value=options.get("value")
            )

        # overrode items per target
        for target, options in manifest.get("target_overrides", {}).items():
            if target != "*" and "TARGET_" + target not in cppdefines:
                continue
            for macro in options.get("target.macros_add", []):
                macro = self._mbed_normalize_macro(macro)
                macros[macro["name"]] = macro
            for key, value in options.items():
                if not key.startswith("target.") and key in macros:
                    macros[key]["value"] = value

        # normalize macro names
        for key, macro in macros.items():
            if not macro["name"]:
                macro["name"] = key
                if "." not in macro["name"]:
                    macro["name"] = "%s.%s" % (manifest.get("name"), macro["name"])
                macro["name"] = re.sub(
                    r"[^a-z\d]+", "_", macro["name"], flags=re.I
                ).upper()
                macro["name"] = "MBED_CONF_" + macro["name"]
            if isinstance(macro["value"], bool):
                macro["value"] = 1 if macro["value"] else 0

        return {macro["name"]: macro["value"] for macro in macros.values()}

    def _mbed_conf_append_macros(self, mbed_config_path, macros):
        lines = []
        with open(mbed_config_path, encoding="utf8") as fp:
            for line in fp.readlines():
                line = line.strip()
                if line == "#endif":
                    lines.append("// PlatformIO Library Dependency Finder (LDF)")
                    lines.extend(
                        [
                            "#define %s %s" % (name, value if value is not None else "")
                            for name, value in macros.items()
                        ]
                    )
                    lines.append("")
                if not line.startswith("#define"):
                    lines.append(line)
                    continue
                tokens = line.split()
                if len(tokens) < 2 or tokens[1] not in macros:
                    lines.append(line)
        lines.append("")
        with open(mbed_config_path, mode="w", encoding="utf8") as fp:
            fp.write("\n".join(lines))


class PlatformIOLibBuilder(LibBuilderBase):
    def load_manifest(self):
        manifest_path = os.path.join(self.path, "library.json")
        if not os.path.isfile(manifest_path):
            return {}
        return ManifestParserFactory.new_from_file(manifest_path).as_dict()

    def _has_arduino_manifest(self):
        return os.path.isfile(os.path.join(self.path, "library.properties"))

    @property
    def include_dir(self):
        if "includeDir" in self._manifest.get("build", {}):
            with fs.cd(self.path):
                return os.path.realpath(self._manifest.get("build").get("includeDir"))
        return LibBuilderBase.include_dir.fget(self)  # pylint: disable=no-member

    @property
    def src_dir(self):
        if "srcDir" in self._manifest.get("build", {}):
            with fs.cd(self.path):
                return os.path.realpath(self._manifest.get("build").get("srcDir"))
        return LibBuilderBase.src_dir.fget(self)  # pylint: disable=no-member

    @property
    def src_filter(self):
        # pylint: disable=no-member
        if "srcFilter" in self._manifest.get("build", {}):
            return self._manifest.get("build").get("srcFilter")
        if self.env["SRC_FILTER"]:
            return self.env["SRC_FILTER"]
        if self._has_arduino_manifest():
            return ArduinoLibBuilder.src_filter.fget(self)
        return LibBuilderBase.src_filter.fget(self)

    @property
    def build_flags(self):
        if "flags" in self._manifest.get("build", {}):
            return self._manifest.get("build").get("flags")
        return LibBuilderBase.build_flags.fget(self)  # pylint: disable=no-member

    @property
    def build_unflags(self):
        if "unflags" in self._manifest.get("build", {}):
            return self._manifest.get("build").get("unflags")
        return LibBuilderBase.build_unflags.fget(self)  # pylint: disable=no-member

    @property
    def extra_script(self):
        if "extraScript" in self._manifest.get("build", {}):
            return self._manifest.get("build").get("extraScript")
        return LibBuilderBase.extra_script.fget(self)  # pylint: disable=no-member

    @property
    def lib_archive(self):
        missing = object()
        global_value = self.env.GetProjectConfig().getraw(
            "env:" + self.env["PIOENV"], "lib_archive", missing
        )
        if global_value != missing:
            return self.env.GetProjectConfig().get(
                "env:" + self.env["PIOENV"], "lib_archive"
            )
        # pylint: disable=no-member
        return self._manifest.get("build", {}).get(
            "libArchive", LibBuilderBase.lib_archive.fget(self)
        )

    @property
    def lib_ldf_mode(self):
        # pylint: disable=no-member
        return self.validate_ldf_mode(
            self._manifest.get("build", {}).get(
                "libLDFMode", LibBuilderBase.lib_ldf_mode.fget(self)
            )
        )

    @property
    def lib_compat_mode(self):
        # pylint: disable=no-member
        return self.validate_compat_mode(
            self._manifest.get("build", {}).get(
                "libCompatMode", LibBuilderBase.lib_compat_mode.fget(self)
            )
        )

    def is_platforms_compatible(self, platforms):
        return util.items_in_list(platforms, self._manifest.get("platforms") or ["*"])

    def is_frameworks_compatible(self, frameworks):
        return util.items_in_list(frameworks, self._manifest.get("frameworks") or ["*"])

    def get_include_dirs(self):
        include_dirs = LibBuilderBase.get_include_dirs(self)

        # backwards compatibility with PlatformIO 2.0
        if (
            "build" not in self._manifest
            and self._has_arduino_manifest()
            and not os.path.isdir(os.path.join(self.path, "src"))
            and os.path.isdir(os.path.join(self.path, "utility"))
        ):
            include_dirs.append(os.path.join(self.path, "utility"))

        for path in self.env.get("CPPPATH", []):
            if path not in self.envorigin.get("CPPPATH", []):
                include_dirs.append(self.env.subst(path))

        return include_dirs


class ProjectAsLibBuilder(LibBuilderBase):
    def __init__(self, env, *args, **kwargs):
        # backup original value, will be reset in base.__init__
        project_src_filter = env.get("SRC_FILTER")
        super(ProjectAsLibBuilder, self).__init__(env, *args, **kwargs)
        self.env["SRC_FILTER"] = project_src_filter

    @property
    def include_dir(self):
        include_dir = self.env.subst("$PROJECT_INCLUDE_DIR")
        return include_dir if os.path.isdir(include_dir) else None

    @property
    def src_dir(self):
        return self.env.subst("$PROJECT_SRC_DIR")

    def get_include_dirs(self):
        include_dirs = []
        project_include_dir = self.env.subst("$PROJECT_INCLUDE_DIR")
        if os.path.isdir(project_include_dir):
            include_dirs.append(project_include_dir)
        for include_dir in LibBuilderBase.get_include_dirs(self):
            if include_dir not in include_dirs:
                include_dirs.append(include_dir)
        return include_dirs

    def get_search_files(self):
        # project files
        items = LibBuilderBase.get_search_files(self)
        # test files
        if "__test" in COMMAND_LINE_TARGETS:
            items.extend(
                [
                    os.path.join("$PROJECT_TEST_DIR", item)
                    for item in self.env.MatchSourceFiles(
                        "$PROJECT_TEST_DIR", "$PIOTEST_SRC_FILTER"
                    )
                ]
            )
        return items

    @property
    def lib_ldf_mode(self):
        mode = LibBuilderBase.lib_ldf_mode.fget(self)  # pylint: disable=no-member
        if not mode.startswith("chain"):
            return mode
        # parse all project files
        return "deep+" if "+" in mode else "deep"

    @property
    def src_filter(self):
        # pylint: disable=no-member
        return self.env.get("SRC_FILTER") or LibBuilderBase.src_filter.fget(self)

    @property
    def dependencies(self):
        return self.env.GetProjectOption("lib_deps", [])

    def process_extra_options(self):
        # skip for project, options are already processed
        pass

    def install_dependencies(self):
        def _is_builtin(spec):
            for lb in self.env.GetLibBuilders():
                if lb.name == spec:
                    return True
            return False

        not_found_specs = []
        for spec in self.dependencies:
            # check if built-in library
            if _is_builtin(spec):
                continue

            found = False
            for storage_dir in self.env.GetLibSourceDirs():
                lm = LibraryPackageManager(storage_dir)
                if lm.get_package(spec):
                    found = True
                    break
            if not found:
                not_found_specs.append(spec)

        did_install = False
        lm = LibraryPackageManager(
            self.env.subst(os.path.join("$PROJECT_LIBDEPS_DIR", "$PIOENV"))
        )
        for spec in not_found_specs:
            try:
                lm.install(spec)
                did_install = True
            except (UnknownPackageError, InternetIsOffline) as e:
                click.secho("Warning! %s" % e, fg="yellow")

        # reset cache
        if did_install:
            DefaultEnvironment().Replace(__PIO_LIB_BUILDERS=None)

    def process_dependencies(self):  # pylint: disable=too-many-branches
        for spec in self.dependencies:
            found = False
            for storage_dir in self.env.GetLibSourceDirs():
                if found:
                    break
                lm = LibraryPackageManager(storage_dir)
                pkg = lm.get_package(spec)
                if not pkg:
                    continue
                for lb in self.env.GetLibBuilders():
                    if pkg.path != lb.path:
                        continue
                    if lb not in self.depbuilders:
                        self.depend_recursive(lb)
                    found = True
                    break
            if found:
                continue

            # look for built-in libraries by a name
            # which don't have package manifest
            for lb in self.env.GetLibBuilders():
                if lb.name != spec:
                    continue
                if lb not in self.depbuilders:
                    self.depend_recursive(lb)
                found = True
                break

    def build(self):
        self._is_built = True  # do not build Project now
        result = LibBuilderBase.build(self)
        self.env.PrependUnique(CPPPATH=self.get_include_dirs())
        return result


def GetLibSourceDirs(env):
    items = env.GetProjectOption("lib_extra_dirs", [])
    items.extend(env["LIBSOURCE_DIRS"])
    return [
        env.subst(fs.expanduser(item) if item.startswith("~") else item)
        for item in items
    ]


def IsCompatibleLibBuilder(env, lb, verbose=int(ARGUMENTS.get("PIOVERBOSE", 0))):
    compat_mode = lb.lib_compat_mode
    if lb.name in env.GetProjectOption("lib_ignore", []):
        if verbose:
            sys.stderr.write("Ignored library %s\n" % lb.path)
        return None
    if compat_mode == "strict" and not lb.is_platforms_compatible(env["PIOPLATFORM"]):
        if verbose:
            sys.stderr.write("Platform incompatible library %s\n" % lb.path)
        return False
    if compat_mode in ("soft", "strict") and not lb.is_frameworks_compatible(
        env.get("PIOFRAMEWORK", [])
    ):
        if verbose:
            sys.stderr.write("Framework incompatible library %s\n" % lb.path)
        return False
    return True


def GetLibBuilders(env):  # pylint: disable=too-many-branches
    if DefaultEnvironment().get("__PIO_LIB_BUILDERS", None) is not None:
        return sorted(
            DefaultEnvironment()["__PIO_LIB_BUILDERS"],
            key=lambda lb: 0 if lb.dependent else 1,
        )

    DefaultEnvironment().Replace(__PIO_LIB_BUILDERS=[])

    verbose = int(ARGUMENTS.get("PIOVERBOSE", 0))
    found_incompat = False

    for storage_dir in env.GetLibSourceDirs():
        storage_dir = os.path.realpath(storage_dir)
        if not os.path.isdir(storage_dir):
            continue
        for item in sorted(os.listdir(storage_dir)):
            lib_dir = os.path.join(storage_dir, item)
            if item == "__cores__" or not os.path.isdir(lib_dir):
                continue
            try:
                lb = LibBuilderFactory.new(env, lib_dir)
            except exception.InvalidJSONFile:
                if verbose:
                    sys.stderr.write(
                        "Skip library with broken manifest: %s\n" % lib_dir
                    )
                continue
            if env.IsCompatibleLibBuilder(lb):
                DefaultEnvironment().Append(__PIO_LIB_BUILDERS=[lb])
            else:
                found_incompat = True

    for lb in env.get("EXTRA_LIB_BUILDERS", []):
        if env.IsCompatibleLibBuilder(lb):
            DefaultEnvironment().Append(__PIO_LIB_BUILDERS=[lb])
        else:
            found_incompat = True

    if verbose and found_incompat:
        sys.stderr.write(
            'More details about "Library Compatibility Mode": '
            "https://docs.platformio.org/page/librarymanager/ldf.html#"
            "ldf-compat-mode\n"
        )

    return DefaultEnvironment()["__PIO_LIB_BUILDERS"]


def ConfigureProjectLibBuilder(env):
    def _correct_found_libs(lib_builders):
        # build full dependency graph
        found_lbs = [lb for lb in lib_builders if lb.dependent]
        for lb in lib_builders:
            if lb in found_lbs:
                lb.search_deps_recursive(lb.get_search_files())
        for lb in lib_builders:
            for deplb in lb.depbuilders[:]:
                if deplb not in found_lbs:
                    lb.depbuilders.remove(deplb)

    def _print_deps_tree(root, level=0):
        margin = "|   " * (level)
        for lb in root.depbuilders:
            title = "<%s>" % lb.name
            pkg = PackageItem(lb.path)
            if pkg.metadata:
                title += " %s" % pkg.metadata.version
            elif lb.version:
                title += " %s" % lb.version
            click.echo("%s|-- %s" % (margin, title), nl=False)
            if int(ARGUMENTS.get("PIOVERBOSE", 0)):
                if pkg.metadata and pkg.metadata.spec.external:
                    click.echo(" [%s]" % pkg.metadata.spec.url, nl=False)
                click.echo(" (", nl=False)
                click.echo(lb.path, nl=False)
                click.echo(")", nl=False)
            click.echo("")
            if lb.depbuilders:
                _print_deps_tree(lb, level + 1)

    project = ProjectAsLibBuilder(env, "$PROJECT_DIR")
    ldf_mode = LibBuilderBase.lib_ldf_mode.fget(project)  # pylint: disable=no-member

    click.echo("LDF: Library Dependency Finder -> https://bit.ly/configure-pio-ldf")
    click.echo(
        "LDF Modes: Finder ~ %s, Compatibility ~ %s"
        % (ldf_mode, project.lib_compat_mode)
    )

    project.install_dependencies()

    lib_builders = env.GetLibBuilders()
    click.echo("Found %d compatible libraries" % len(lib_builders))

    click.echo("Scanning dependencies...")
    project.search_deps_recursive()

    if ldf_mode.startswith("chain") and project.depbuilders:
        _correct_found_libs(lib_builders)

    if project.depbuilders:
        click.echo("Dependency Graph")
        _print_deps_tree(project)
    else:
        click.echo("No dependencies")

    return project


def exists(_):
    return True


def generate(env):
    env.AddMethod(GetLibSourceDirs)
    env.AddMethod(IsCompatibleLibBuilder)
    env.AddMethod(GetLibBuilders)
    env.AddMethod(ConfigureProjectLibBuilder)
    return env



#copy from source https://github.com/securisec/ripgrepy
#thanks for the author
import os
from shutil import which
from subprocess import getoutput
from json import dumps, loads
from functools import wraps
from timeit import default_timer
from pkg_resources import get_distribution
import logging

__version__ = get_distribution("ripgrepy").version


def _logger(func):
    """
    Logger decorator
    """

    @wraps(func)
    def l(*args, **kwargs):
        try:
            start = default_timer()
            o = func(*args, **kwargs)
            end = default_timer()
            rt = str(round(end - start, 4)) + " seconds"
            logging.debug(f"{func.__name__} runtime {rt}")
            return o
        except:
            logging.exception("")
            raise

    return l


class RipGrepNotFound(Exception):
    pass


class RipGrepOut(object):
    def __init__(self, _output, command):
        self._output = _output
        self.command = command

    @property
    @_logger
    def as_dict(self) -> list:
        """
        Returns an array of objects with the match. The objects include
        file path, line number and matched value. This is in addition to the
        --json that can be passed to ripgrep and is designed for simple ripgrep use

        :return: Array of matched objects
        :rtype: list

        The following is an example of the dict output.

        >>> [{'data': {'absolute_offset': 12,
        >>>   'line_number': 3,
        >>>   'lines': {'text': 'teststring\\n'},
        >>>   'path': {'text': '/tmp/test/test.lol'},
        >>>   'submatches': [{'end': 4, 'match': {'text': 'test'}, 'start': 0}]},
        >>> 'type': 'match'}]
        """
        if "--json" not in self.command:
            raise TypeError("To use as_dict, use the json() method")
        out = self._output.splitlines()
        holder = []
        for line in out:
            data = loads(line)
            if data["type"] == "match":
                holder.append(data)
        return holder

    @property
    @_logger
    def as_json(self) -> str:
        """
        Returns the output as a JSON object. This is in addition to the
        --json that can be passed to ripgrep and is designed for simple ripgrep use

        :return: JSON object
        :rtype: str
        """
        if "--json" not in self.command:
            raise TypeError("To use as_dict, use the json() method")
        out = self._output.splitlines()
        holder = []
        for line in out:
            data = loads(line)
            if data["type"] == "match":
                holder.append(data)
        return dumps(holder)

    @property
    @_logger
    def as_string(self) -> str:
        """
        Returns stdout from ripgrep

        :return: Stdout of ripgrep
        :rtype: str
        """
        return self._output

    def __repr__(self):
        return str(self._output)


class Ripgrepy(object):
    """
    The main class for Ripgrepy.

    :param regex_pattern: A regular expression used for searching
    :type regex_pattern: str
    :param path: A file or directory to search. Directories are searched
           recursively. Paths specified explicitly on the command line
           override glob and ignore rules
    :type path: str
    :param rg_path: Path to ripgrep. Defaults to $PATH
    :type rg_path: str
    :raises RipGrepNotFound: Error if path to ripgrep could not be resolved
    """

    def __init__(self, regex_pattern: str, path: str, rg_path: str = "rg"):
        self.regex_pattern = f'"{regex_pattern}"'
        self.path = os.path.expanduser(path)
        self._output = None
        self._rg_path = rg_path
        #: The ripgreg command that will be executed
        self.command = [self._rg_path]

        if which(self._rg_path) is None:
            raise RipGrepNotFound("ripgrep not found")

        # short syntax mapping
        #: Short syntax for byte_offset
        self.b = self.byte_offset
        #: Short syntax for case_sensitive
        self.s = self.case_sensitive
        #: Short syntax for encoding
        self.E = self.encoding
        #: Short syntax for file
        self.f = self.file
        #: Short syntax for files_with_matches
        self.l = self.files_with_matches
        #: Short syntax for fixed_strings
        self.F = self.fixed_strings
        #: Short syntax for follow
        self.L = self.follow
        #: Short syntax for glob
        self.g = self.glob
        #: Short syntax for ignore_case
        self.i = self.ignore_case
        #: Short syntax for invert_match
        self.v = self.invert_match
        #: Short syntax for line_number
        self.n = self.line_number
        #: Short syntax for after_context
        self.A = self.after_context
        #: Short syntax for before_context
        self.B = self.before_context
        #: Short syntax for context
        self.C = self.context
        #: Short syntax for line_regexp
        self.x = self.line_regexp
        #: Short syntax for max_columns
        self.M = self.max_columns
        #: Short syntax for max_count
        self.m = self.max_count
        #: Short syntax for multiline
        self.U = self.multiline
        #: Short syntax for no_filename
        self.I = self.no_filename
        #: Short syntax for no_line_number
        self.N = self.no_line_number
        #: Short syntax for only_matching
        self.o = self.only_matching
        #: Short syntax for pcre2
        self.P = self.pcre2
        #: Short syntax for pretty
        self.p = self.pretty
        #: Short syntax for quite
        self.q = self.quite
        #: Short syntax for regexp
        self.e = self.regexp
        #: Short syntax for replace
        self.r = self.replace
        #: Short syntax for search_zip
        self.z = self.search_zip
        #: Short syntax for smart_case
        self.S = self.smart_case
        #: Short syntax for text
        self.a = self.text
        #: Short syntax for threads
        self.j = self.threads
        #: Short syntax for type_not
        self.T = self.type_not
        #: Short syntax for unrestricted
        self.u = self.unrestricted
        #: Short syntax for with_filename
        self.H = self.with_filename
        #: Short syntax for word_regexp
        self.w = self.word_regexp
        #: Alias to run
        self.run_rg = self.run

    @_logger
    def run(self):# -> RipgrepOut
        """
        Returns an instace of the Ripgrepy object

        :return: self
        :rtype: RipgrepOut
        """
        self.command.append(self.regex_pattern)
        self.command.append(self.path)
        self.command = " ".join(self.command)
        self._output = getoutput(self.command)
        return RipGrepOut(self._output, self.command)

    @_logger
    def error_msg(self):#-> Ripgrepy
        """
        Returns any stderr that was generated

        :return: String of error
        :rtype: Ripgrepy
        """
        return self.error

    @_logger
    def after_context(self, number: int):#-> Ripgrepy
        """
        Show NUM lines after each match.

        This overrides the --context flag.

        :param number: Number of lines to show
        :type number: int
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f"--after-context {number}")
        return self

    @_logger
    def before_context(self, number: int):# -> Ripgrepy
        """
        Show NUM lines before each match.

        This overrides the --context flag.

        :param number: Number of lines to show
        :type number: int
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f"--before-context {number}")
        return self

    @_logger
    def context(self, number: int):# -> Ripgrepy
        """
        Show NUM lines before and after each match. This is equivalent to
        providing both the -B/--before-context and -A/--after-context flags
        with the same value.

        This overrides both the -B/--before-context and -A/--after-context
        flags.


        :param number: Number of lines to show
        :type number: int
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f"--context {number}")
        return self

    @_logger
    def binary(self):# -> Ripgrepy
        """
        Enabling this flag will cause ripgrep to search binary files. By
        default, ripgrep attempts to automatically skip binary files in
        order to improve the relevance of results and make the search
        faster.

        Binary files are heuristically detected based on whether they
        contain a NUL byte or not. By default (without this flag set), once
        a NUL byte is seen, ripgrep will stop searching the file. Usually,
        NUL bytes occur in the beginning of most binary files. If a NUL
        byte occurs after a match, then ripgrep will still stop searching
        the rest of the file, but a warning will be printed.

        In contrast, when this flag is provided, ripgrep will continue
        searching a file even if a NUL byte is found. In particular, if a
        NUL byte is found then ripgrep will continue searching until either
        a match is found or the end of the file is reached, whichever comes
        sooner. If a match is found, then ripgrep will stop and print a
        warning saying that the search stopped prematurely.

        If you want ripgrep to search a file without any special NUL byte
        handling at all (and potentially print binary data to stdout), then
        you should use the -a/--text flag.

        The --binary flag is a flag for controlling ripgrep's automatic
        filtering mechanism. As such, it does not need to be used when
        searching a file explicitly or when searching stdin. That is, it is
        only applicable when recursively searching a directory.

        Note that when the -u/--unrestricted flag is provided for a third
        time, then this flag is automatically enabled.

        This flag can be disabled with --no-binary. It overrides the
        -a/--text flag.
        """
        self.command.append("--binary")
        return self

    @_logger
    def auto_hybrid_regex(self):# -> Ripgrepy
        """
        When this flag is used, ripgrep will dynamically choose between
        supported regex engines depending on the features used in a
        pattern. When ripgrep chooses a regex engine, it applies that
        choice for every regex provided to ripgrep (e.g., via multiple
        -e/--regexp or -f/--file flags).

        As an example of how this flag might behave, ripgrep will attempt
        to use its default finite automata based regex engine whenever the
        pattern can be successfully compiled with that regex engine. If
        PCRE2 is enabled and if the pattern given could not be compiled
        with the default regex engine, then PCRE2 will be automatically
        used for searching. If PCRE2 isn't available, then this flag has no
        effect because there is only one regex engine to choose from.

        In the future, ripgrep may adjust its heuristics for how it decides
        which regex engine to use. In general, the heuristics will be
        limited to a static analysis of the patterns, and not to any
        specific runtime behavior observed while searching files.

        The primary downside of using this flag is that it may not always
        be obvious which regex engine ripgrep uses, and thus, the match
        semantics or performance profile of ripgrep may subtly and
        unexpectedly change. However, in many cases, all regex engines will
        agree on what constitutes a match and it can be nice to
        transparently support more advanced regex features like look-around
        and backreferences without explicitly needing to enable them.

        This flag can be disabled with --no-auto-hybrid-regex.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--auto-hybrid-regex")
        return self

    @_logger
    def block_buffered(self):# -> Ripgrepy
        """
        When enabled, ripgrep will use block buffering. That is, whenever a
        matching line is found, it will be written to an in-memory buffer
        and will not be written to stdout until the buffer reaches a
        certain size. This is the default when ripgrep's stdout is
        redirected to a pipeline or a file. When ripgrep's stdout is
        connected to a terminal, line buffering will be used. Forcing block
        buffering can be useful when dumping a large amount of contents to
        a terminal.

        Forceful block buffering can be disabled with --no-block-buffered.
        Note that using --no-block-buffered causes ripgrep to revert to its
        default behavior of automatically detecting the buffering strategy.
        To force line buffering, use the --line-buffered flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--block-buffered")
        return self

    @_logger
    def byte_offset(self):# -> Ripgrepy
        """
        Print the 0-based byte offset within the input file before each
        line of output. If -o (--only-matching) is specified, print the
        offset of the matching part itself.

        If ripgrep does transcoding, then the byte offset is in terms of
        the the result of transcoding and not the original data. This
        applies similarly to another transformation on the source, such as
        decompression or a --pre filter. Note that when the PCRE2 regex
        engine is used, then UTF-8 transcoding is done by default.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--byte-offset")
        return self

    @_logger
    def case_sensitive(self):# -> Ripgrepy
        """
        Search case sensitively.

        This overrides the -i/--ignore-case and -S/--smart-case flags.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--case-sensitive")
        return self

    @_logger
    def count_matches(self):# -> Ripgrepy
        """
        This flag suppresses normal output and shows the number of
        individual matches of the given patterns for each file searched.
        Each file containing matches has its path and match count printed
        on each line. Note that this reports the total number of individual
        matches and not the number of lines that match.

        If only one file is given to ripgrep, then only the count is
        printed if there is a match. The --with-filename flag can be used
        to force printing the file path in this case.

        This overrides the --count flag. Note that when --count is combined
        with --only-matching, then ripgrep behaves as if --count-matches
        was given.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--count-matches")
        return self

    @_logger
    def crlf(self):# -> Ripgrepy
        """
        When enabled, ripgrep will treat CRLF (\\r\\n) as a line terminator
        instead of just \\n.

        Principally, this permits $ in regex patterns to match just before
        CRLF instead of just before LF. The underlying regex engine may not
        support this natively, so ripgrep will translate all instances of $
        to (?:\\r??$). This may produce slightly different than desired
        match offsets. It is intended as a work-around until the regex
        engine supports this natively.

        CRLF support can be disabled with --no-crlf.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--crlf")
        return self

    @_logger
    def debug(self):# -> Ripgrepy
        """
        Show debug messages. Please use this when filing a bug report.

        The --debug flag is generally useful for figuring out why ripgrep
        skipped searching a particular file. The debug messages should
        mention all files skipped and why they were skipped.

        To get even more debug output, use the --trace flag, which implies
        --debug along with additional trace data. With --trace, the output
        could be quite large and is generally more useful for development.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--debug")
        return self

    @_logger
    def dfa_size_limit(self, num_suffix: int):# -> Ripgrepy
        """
        The upper size limit of the regex DFA. The default limit is 10M.
        This should only be changed on very large regex inputs where the
        (slower) fallback regex engine may otherwise be used if the limit
        is reached.

        The argument accepts the same size suffixes as allowed in with the
        --max-filesize flag.

        :param num_suffix: size suffixes
        :type num_suffix: int
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--dfa-size-limit "{num_suffix}"')
        return self

    @_logger
    def encoding(self, encoding: str):# -> Ripgrepy
        """
        Specify the text encoding that ripgrep will use on all files
        searched. The default value is auto, which will cause ripgrep to do
        a best effort automatic detection of encoding on a per-file basis.
        Automatic detection in this case only applies to files that begin
        with a UTF-8 or UTF-16 byte-order mark (BOM). No other automatic
        detection is performed. One can also specify none which will then
        completely disable BOM sniffing and always result in searching the
        raw bytes, including a BOM if it's present, regardless of its
        encoding.

        Other supported values can be found in the list of labels here:
        https://encoding.spec.whatwg.org/#concept-encoding-get

        For more details on encoding and how ripgrep deals with it, see
        GUIDE.md.

        This flag can be disabled with --no-encoding.

        :param encoding: encoding
        :type encoding: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--encoding "{encoding}"')
        return self

    @_logger
    def file(self, pattern: str):# -> Ripgrepy
        """
        Search for patterns from the given file, with one pattern per line.
        When this flag is used multiple times or in combination with the
        -e/--regexp flag, then all patterns provided are searched. Empty
        pattern lines will match all input lines, and the newline is not
        counted as part of the pattern.

        A line is printed if and only if it matches at least one of the
        patterns.

        :param pattern: pattern
        :type pattern: str
        :return: [description]
        :rtype: Ripgrepy
        """
        self.command.append(f'--file "{pattern}"')
        return self

    @_logger
    def files(self):# -> Ripgrepy
        """
        Print each file that would be searched without actually performing
        the search. This is useful to determine whether a particular file
        is being searched or not.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--files")
        return self

    @_logger
    def files_with_matches(self):# -> Ripgrepy
        """
        Only print the paths with at least one match.

        This overrides --files-without-match.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--files-with-matches")
        return self

    @_logger
    def files_without_match(self):# -> Ripgrepy
        """
        Only print the paths that contain zero matches. This
        inverts/negates the --files-with-matches flag.

        This overrides --files-with-matches.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--files-without-match")
        return self

    @_logger
    def fixed_strings(self):# -> Ripgrepy
        """
        Treat the pattern as a literal string instead of a regular
        expression. When this flag is used, special regular expression meta
        characters such as .(){}*+ do not need to be escaped.

        This flag can be disabled with --no-fixed-strings.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--fixed-strings")
        return self

    @_logger
    def follow(self):# -> Ripgrepy
        """
        When this flag is enabled, ripgrep will follow symbolic links while
        traversing directories. This is disabled by default. Note that
        ripgrep will check for symbolic link loops and report errors if it
        finds one.

        This flag can be disabled with --no-follow.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--follow")
        return self

    @_logger
    def glob(self, glob_pattern: str):# -> Ripgrepy
        """
        Include or exclude files and directories for searching that match
        the given glob. This always overrides any other ignore logic.
        Multiple glob flags may be used. Globbing rules match .gitignore
        globs. Precede a glob with a ! to exclude it.

        :param glob_pattern: Glob pattern
        :type glob_pattern: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--glob "{glob_pattern}"')
        return self

    @_logger
    def hidden(self):# -> Ripgrepy
        """
        Search hidden files and directories. By default, hidden files and
        directories are skipped. Note that if a hidden file or a directory
        is whitelisted in an ignore file, then it will be searched even if
        this flag isn't provided.

        This flag can be disabled with --no-hidden.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--hidden")
        return self

    @_logger
    def iglob(self, glob_pattern: str):# -> Ripgrepy
        """
        Include or exclude files and directories for searching that match
        the given glob. This always overrides any other ignore logic.
        Multiple glob flags may be used. Globbing rules match .gitignore
        globs. Precede a glob with a ! to exclude it. Globs are matched
        case insensitively.

        :param glob_pattern: Glob pattern
        :type glob_pattern: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--iglob "{glob_pattern}"')
        return self

    @_logger
    def ignore_case(self):# -> Ripgrepy
        """
        When this flag is provided, the given patterns will be searched
        case insensitively. The case insensitivity rules used by ripgrep
        conform to Unicode's "simple" case folding rules.

        This flag overrides -s/--case-sensitive and -S/--smart-case.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--ignore-case")
        return self

    @_logger
    def ignore_file(self, path: str):# -> Ripgrepy
        """
        Specifies a path to one or more .gitignore format rules files.
        These patterns are applied after the patterns found in .gitignore
        and .ignore are applied and are matched relative to the current
        working directory. Multiple additional ignore files can be
        specified by using the --ignore-file flag several times. When
        specifying multiple ignore files, earlier files have lower
        precedence than later files.

        If you are looking for a way to include or exclude files and
        directories directly on the command line, then used -g instead.

        :param path: File path
        :type path: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--ignore-file "{path}"')
        return self

    @_logger
    def ignore_file_case_insensitive(self):# -> Ripgrepy
        """
        Process ignore files (.gitignore, .ignore, etc.) case
        insensitively. Note that this comes with a performance penalty and
        is most useful on case insensitive file systems (such as Windows).

        This flag can be disabled with the
        --no-ignore-file-case-insensitive flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--ignore-file-case-insensitive")
        return self

    @_logger
    def invert_match(self):# -> Ripgrepy
        """
        Invert matching. Show lines that do not match the given patterns.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--invert-match")
        return self

    @_logger
    def json(self):# -> Ripgrepy
        """
        Enable printing results in a JSON Lines format.

        When this flag is provided, ripgrep will emit a sequence of
        messages, each encoded as a JSON object, where there are five
        different message types:

        begin - A message that indicates a file is being searched and
        contains at least one match.

        end - A message the indicates a file is done being searched. This
        message also include summary statistics about the search for a
        particular file.

        match - A message that indicates a match was found. This includes
        the text and offsets of the match.

        context - A message that indicates a contextual line was found.
        This includes the text of the line, along with any match
        information if the search was inverted.

        summary - The final message emitted by ripgrep that contains
        summary statistics about the search across all files.

        Since file paths or the contents of files are not guaranteed to be
        valid UTF-8 and JSON itself must be representable by a Unicode
        encoding, ripgrep will emit all data elements as objects with one
        of two keys: text or bytes.  text is a normal JSON string when the
        data is valid UTF-8 while bytes is the base64 encoded contents of
        the data.

        The JSON Lines format is only supported for showing search results.
        It cannot be used with other flags that emit other types of output,
        such as --files, --files-with-matches, --files-without-match,
        --count or --count-matches. ripgrep will report an error if any of
        the aforementioned flags are used in concert with --json.

        Other flags that control aspects of the standard output such as
        --only-matching, --heading, --replace, --max-columns, etc., have no
        effect when --json is set.

        A more complete description of the JSON format used can be found
        `here: <https://docs.rs/grep-printer/*/grep_printer/struct.JSON.html>__`

        The JSON Lines format can be disabled with --no-json.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--json")
        return self

    @_logger
    def line_buffered(self):# -> Ripgrepy
        """
        When enabled, ripgrep will use line buffering. That is, whenever a
        matching line is found, it will be flushed to stdout immediately.
        This is the default when ripgrep's stdout is connected to a
        terminal, but otherwise, ripgrep will use block buffering, which is
        typically faster. This flag forces ripgrep to use line buffering
        even if it would otherwise use block buffering. This is typically
        useful in shell pipelines, e.g., tail -f something.log | rg foo
        --line-buffered | rg bar.

        Forceful line buffering can be disabled with --no-line-buffered.
        Note that using --no-line-buffered causes ripgrep to revert to its
        default behavior of automatically detecting the buffering strategy.
        To force block buffering, use the --block-buffered flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--line-buffered")
        return self

    @_logger
    def line_number(self):# -> Ripgrepy
        """
        Show line numbers (1-based). This is enabled by default when
        searching in a terminal.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--line-number")
        return self

    @_logger
    def line_regexp(self):# -> Ripgrepy
        """
        Only show matches surrounded by line boundaries. This is equivalent
        to putting ^...$ around all of the search patterns. In other words,
        this only prints lines where the entire line participates in a
        match.

        This overrides the --word-regexp flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--line-regexp")
        return self

    @_logger
    def max_columns(self, num: int):# -> Ripgrepy
        """
        Don't print lines longer than this limit in bytes. Longer lines are
        omitted, and only the number of matches in that line is printed.

        When this flag is omitted or is set to 0, then it has no effect.

        :param num: Number of columns
        :type num: int
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f"--max-columns {num}")
        return self

    @_logger
    def max_columns_preview(self):# -> Ripgrepy
        """
        When the --max-columns flag is used, ripgrep will by default
        completely replace any line that is too long with a message
        indicating that a matching line was removed. When this flag is
        combined with --max-columns, a preview of the line (corresponding
        to the limit size) is shown instead, where the part of the line
        exceeding the limit is not shown.

        If the --max-columns flag is not set, then this has no effect.

        This flag can be disabled with --no-max-columns-preview.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--max-columns-preview")
        return self

    @_logger
    def max_count(self, num: int):# -> Ripgrepy
        """
        Limit the number of matching lines per file searched to NUM.

        :param num: Number
        :type num: int
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f"--max-count {num}")
        return self

    @_logger
    def max_depth(self, num: int):# -> Ripgrepy
        """
        Limit the depth of directory traversal to NUM levels beyond the
        paths given. A value of zero only searches the explicitly given
        paths themselves.

        For example, rg --max-depth 0 dir/ is a no-op because dir/ will not
        be descended into.  rg --max-depth 1 dir/ will search only the
        direct children of dir.

        :param num: Number
        :type num: int
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f"--max-depth {num}")
        return self

    @_logger
    def max_filesize(self, num_suffix: str):# -> Ripgrepy
        """
        Ignore files larger than NUM in size. This does not apply to
        directories.

        The input format accepts suffixes of K, M or G which correspond to
        kilobytes, megabytes and gigabytes, respectively. If no suffix is
        provided the input is treated as bytes.

        Examples: --max-filesize 50K or --max-filesize 80M

        :param num_suffix: number plus suffic
        :type num_suffix: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--max-filesize "{num_suffix}"')
        return self

    @_logger
    def mmap(self):# -> Ripgrepy
        """
        Search using memory maps when possible. This is enabled by default
        when ripgrep thinks it will be faster.

        Memory map searching doesn't currently support all options, so if
        an incompatible option (e.g., --context) is given with --mmap, then
        memory maps will not be used.

        Note that ripgrep may abort unexpectedly when --mmap if it searches
        a file that is simultaneously truncated.

        This flag overrides --no-mmap.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--mmap")
        return self

    @_logger
    def multiline(self):# -> Ripgrepy
        """
        Enable matching across multiple lines.

        When multiline mode is enabled, ripgrep will lift the restriction
        that a match cannot include a line terminator. For example, when
        multiline mode is not enabled (the default), then the regex \p{any}
        will match any Unicode codepoint other than \\n. Similarly, the
        regex \\n is explicitly forbidden, and if you try to use it, ripgrep
        will return an error. However, when multiline mode is enabled,
        \p{any} will match any Unicode codepoint, including \\n, and regexes
        like \\n are permitted.

        An important caveat is that multiline mode does not change the
        match semantics of .. Namely, in most regex matchers, a .  will by
        default match any character other than \\n, and this is true in
        ripgrep as well. In order to make .  match \\n, you must enable the
        "dot all" flag inside the regex. For example, both (?s).  and
        (?s:.)  have the same semantics, where .  will match any character,
        including \\n. Alternatively, the --multiline-dotall flag may be
        passed to make the "dot all" behavior the default. This flag only
        applies when multiline search is enabled.

        There is no limit on the number of the lines that a single match
        can span.

        WARNING: Because of how the underlying regex engine works,
        multiline searches may be slower than normal line-oriented
        searches, and they may also use more memory. In particular, when
        multiline mode is enabled, ripgrep requires that each file it
        searches is laid out contiguously in memory (either by reading it
        onto the heap or by memory-mapping it). Things that cannot be
        memory-mapped (such as stdin) will be consumed until EOF before
        searching can begin. In general, ripgrep will only do these things
        when necessary. Specifically, if the --multiline flag is provided
        but the regex does not contain patterns that would match \\n
        characters, then ripgrep will automatically avoid reading each file
        into memory before searching it. Nevertheless, if you only care
        about matches spanning at most one line, then it is always better
        to disable multiline mode.

        This flag can be disabled with --no-multiline.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--multiline")
        return self

    @_logger
    def multiline_dotall(self):# -> Ripgrepy
        """
        This flag enables "dot all" in your regex pattern, which causes .
        to match newlines when multiline searching is enabled. This flag
        has no effect if multiline searching isn't enabled with the
        --multiline flag.

        Normally, a .  will match any character except newlines. While this
        behavior typically isn't relevant for line-oriented matching (since
        matches can span at most one line), this can be useful when
        searching with the -U/--multiline flag. By default, the multiline
        mode runs without this flag.

        This flag is generally intended to be used in an alias or your
        ripgrep config file if you prefer "dot all" semantics by default.
        Note that regardless of whether this flag is used, "dot all"
        semantics can still be controlled via inline flags in the regex
        pattern itself, e.g., (?s:.)  always enables "dot all" whereas
        (?-s:.)  always disables "dot all".

        This flag can be disabled with --no-multiline-dotall.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--multiline-dotall")
        return self

    @_logger
    def no_config(self):# -> Ripgrepy
        """
        Never read configuration files. When this flag is present, ripgrep
        will not respect the RIPGREP_CONFIG_PATH environment variable.

        If ripgrep ever grows a feature to automatically read configuration
        files in pre-defined locations, then this flag will also disable
        that behavior as well.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-config")
        return self

    @_logger
    def no_filename(self):# -> Ripgrepy
        """
        Never print the file path with the matched lines. This is the
        default when ripgrep is explicitly instructed to search one file or
        stdin.

        This flag overrides --with-filename.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-filename")
        return self

    @_logger
    def no_heading(self):# -> Ripgrepy
        """
        Don't group matches by each file. If --no-heading is provided in
        addition to the -H/--with-filename flag, then file paths will be
        printed as a prefix for every matched line. This is the default
        mode when not printing to a terminal.

        This overrides the --heading flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-heading")
        return self

    @_logger
    def no_ignore(self):# -> Ripgrepy
        """
        Don't respect ignore files (.gitignore, .ignore, etc.). This
        implies --no-ignore-parent, --no-ignore-dot and --no-ignore-vcs.

        This flag can be disabled with the --ignore flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-ignore")
        return self

    @_logger
    def no_ignore_dot(self):# -> Ripgrepy
        """
        Don't respect .ignore files.

        This flag can be disabled with the --ignore-dot flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-ignore-dot")
        return self

    @_logger
    def no_ignore_global(self):# -> Ripgrepy
        """
        Don't respect ignore files that come from "global" sources such as
        git's core.excludesFile configuration option (which defaults to
        $HOME/.config/git/ignore).

        This flag can be disabled with the --ignore-global flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-ignore-global")
        return self

    @_logger
    def no_ignore_messages(self):# -> Ripgrepy
        """
        Suppresses all error messages related to parsing ignore files such
        as .ignore or .gitignore.

        This flag can be disabled with the --ignore-messages flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-ignore-messages")
        return self

    @_logger
    def no_ignore_parent(self):# -> Ripgrepy
        """
        Don't respect ignore files (.gitignore, .ignore, etc.) in parent
        directories.

        This flag can be disabled with the --ignore-parent flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-ignore-parent")
        return self

    @_logger
    def no_ignore_vcs(self):# -> Ripgrepy
        """
        Don't respect version control ignore files (.gitignore, etc.). This
        implies --no-ignore-parent for VCS files. Note that .ignore files
        will continue to be respected.

        This flag can be disabled with the --ignore-vcs flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-ignore-vcs")
        return self

    @_logger
    def no_line_number(self):# -> Ripgrepy
        """
        Suppress line numbers. This is enabled by default when not
        searching in a terminal.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-line-number")
        return self

    @_logger
    def no_messages(self):# -> Ripgrepy
        """
        Suppress all error messages related to opening and reading files.
        Error messages related to the syntax of the pattern given are still
        shown.

        This flag can be disabled with the --messages flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-messages")
        return self

    @_logger
    def no_mmap(self):# -> Ripgrepy
        """
        Never use memory maps, even when they might be faster.

        This flag overrides --mmap.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-mmap")
        return self

    @_logger
    def no_pcre2_unicode(self):# -> Ripgrepy
        """
        When PCRE2 matching is enabled, this flag will disable Unicode
        mode, which is otherwise enabled by default. If PCRE2 matching is
        not enabled, then this flag has no effect.

        When PCRE2's Unicode mode is enabled, several different types of
        patterns become Unicode aware. This includes \\b, \B, \w, \W, \d,
        \D, \s and \S. Similarly, the .  meta character will match any
        Unicode codepoint instead of any byte. Caseless matching will also
        use Unicode simple case folding instead of ASCII-only case
        insensitivity.

        Unicode mode in PCRE2 represents a critical trade off in the user
        experience of ripgrep. In particular, unlike the default regex
        engine, PCRE2 does not support the ability to search possibly
        invalid UTF-8 with Unicode features enabled. Instead, PCRE2
        requires that everything it searches when Unicode mode is enabled
        is valid UTF-8. (Or valid UTF-16/UTF-32, but for the purposes of
        ripgrep, we only discuss UTF-8.) This means that if you have
        PCRE2's Unicode mode enabled and you attempt to search invalid
        UTF-8, then the search for that file will halt and print an error.
        For this reason, when PCRE2's Unicode mode is enabled, ripgrep will
        automatically "fix" invalid UTF-8 sequences by replacing them with
        the Unicode replacement codepoint.

        If you would rather see the encoding errors surfaced by PCRE2 when
        Unicode mode is enabled, then pass the --no-encoding flag to
        disable all transcoding.

        Related flags: --pcre2

        This flag can be disabled with --pcre2-unicode.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-pcre2-unicode")
        return self

    @_logger
    def null(self):# -> Ripgrepy
        """
        Whenever a file path is printed, follow it with a NUL byte. This
        includes printing file paths before matches, and when printing a
        list of matching files such as with --count, --files-with-matches
        and --files. This option is useful for use with xargs.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--null")
        return self

    @_logger
    def null_data(self):# -> Ripgrepy
        """
        Enabling this option causes ripgrep to use NUL as a line terminator
        instead of the default of \\n.

        This is useful when searching large binary files that would
        otherwise have very long lines if \\n were used as the line
        terminator. In particular, ripgrep requires that, at a minimum,
        each line must fit into memory. Using NUL instead can be a useful
        stopgap to keep memory requirements low and avoid OOM (out of
        memory) conditions.

        This is also useful for processing NUL delimited data, such as that
        emitted when using ripgrep's -0/--null flag or find's --print0
        flag.

        Using this flag implies -a/--text.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--null-data")
        return self

    @_logger
    def one_file_system(self):# -> Ripgrepy
        """
        When enabled, ripgrep will not cross file system boundaries
        relative to where the search started from.

        Note that this applies to each path argument given to ripgrep. For
        example, in the command rg --one-file-system /foo/bar /quux/baz,
        ripgrep will search both /foo/bar and /quux/baz even if they are on
        different file systems, but will not cross a file system boundary
        when traversing each path's directory tree.

        This is similar to find's -xdev or -mount flag.

        This flag can be disabled with --no-one-file-system.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--one-file-system")
        return self

    @_logger
    def only_matching(self):# -> Ripgrepy
        """
        Print only the matched (non-empty) parts of a matching line, with
        each such part on a separate output line.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--only-matching")
        return self

    @_logger
    def passthru(self):# -> Ripgrepy
        """
        Print both matching and non-matching lines.

        Another way to achieve a similar effect is by modifying your
        pattern to match the empty string. For example, if you are
        searching using rg foo then using rg "^|foo" instead will emit
        every line in every file searched, but only occurrences of foo will
        be highlighted. This flag enables the same behavior without needing
        to modify the pattern.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--passthru")
        return self

    @_logger
    def path_seprator(self, separator: str):# -> Ripgrepy
        """
        Set the path separator to use when printing file paths. This
        defaults to your platform's path separator, which is / on Unix and
        \ on Windows. This flag is intended for overriding the default when
        the environment demands it (e.g., cygwin). A path separator is
        limited to a single byte.

        :param separator: separator
        :type separator: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--path-separator "{separator}"')
        return self

    @_logger
    def pcre2(self):# -> Ripgrepy
        """
        When this flag is present, ripgrep will use the PCRE2 regex engine
        instead of its default regex engine.

        This is generally useful when you want to use features such as
        look-around or backreferences.

        Note that PCRE2 is an optional ripgrep feature. If PCRE2 wasn't
        included in your build of ripgrep, then using this flag will result
        in ripgrep printing an error message and exiting.

        Related flags: --no-pcre2-unicode

        This flag can be disabled with --no-pcre2.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--pcre2")
        return self

    @_logger
    def pcre2_version(self):# -> Ripgrepy
        """
        When this flag is present, ripgrep will print the version of PCRE2
        in use, along with other information, and then exit. If PCRE2 is
        not available, then ripgrep will print an error message and exit
        with an error code.

        :return: self
        :rtype: Ripgrepy
        """
        self.regex_pattern = ""
        self.command.append("--pcre2-version")
        return self

    @_logger
    def pre(self, command: str):# -> Ripgrepy
        """
        For each input FILE, search the standard output of COMMAND FILE
        rather than the contents of FILE. This option expects the COMMAND
        program to either be an absolute path or to be available in your
        PATH. Either an empty string COMMAND or the --no-pre flag will
        disable this behavior.

        WARNING: When this flag is set, ripgrep will unconditionally spawn a
        process for every file that is searched. Therefore, this can incur an
        unnecessarily large performance penalty if you don't otherwise need the
        flexibility offered by this flag. One possible mitigation to this is to use
        the '--pre-glob' flag to limit which files a preprocessor is run with.

        A preprocessor is not run when ripgrep is searching stdin.

        When searching over sets of files that may require one of several
        decoders as preprocessors, COMMAND should be a wrapper program or
        script which first classifies FILE based on magic numbers/content
        or based on the FILE name and then dispatches to an appropriate
        preprocessor. Each COMMAND also has its standard input connected to
        FILE for convenience.

        For example, a shell script for COMMAND might look like:

        >>>    case "$1" in
        >>>    *.pdf)
        >>>        exec pdftotext "$1" -
        >>>        ;;
        >>>    *)
        >>>        case $(file "$1") in
        >>>        *Zstandard*)
        >>>            exec pzstd -cdq
        >>>            ;;
        >>>        *)
        >>>            exec cat
        >>>            ;;
        >>>        esac
        >>>        ;;
        >>>    esac

        The above script uses pdftotext to convert a PDF file to plain
        text. For all other files, the script uses the file utility to
        sniff the type of the file based on its contents. If it is a
        compressed file in the Zstandard format, then pzstd is used to
        decompress the contents to stdout.

        This overrides the -z/--search-zip flag.

        :param command: command to execute
        :type command: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--pre "{command}"')
        return self

    @_logger
    def pre_glob(self, glob: str):# -> Ripgrepy
        """
        This flag works in conjunction with the --pre flag. Namely, when
        one or more --pre-glob flags are given, then only files that match
        the given set of globs will be handed to the command specified by
        the --pre flag. Any non-matching files will be searched without
        using the preprocessor command.

        This flag is useful when searching many files with the --pre flag.
        Namely, it permits the ability to avoid process overhead for files
        that don't need preprocessing. For example, given the following
        shell script, pre-pdftotext:

        >>>    #!/bin/sh
        >>>    pdftotext "$1" -

        then it is possible to use --pre pre-pdftotext --pre-glob '\*.pdf'
        to make it so ripgrep only executes the pre-pdftotext command on
        files with a .pdf extension.

        Multiple --pre-glob flags may be used. Globbing rules match
        .gitignore globs. Precede a glob with a ! to exclude it.

        This flag has no effect if the --pre flag is not used.

        :param glob: Glob pattern
        :type glob: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--pre-glob "{glob}"')
        return self

    @_logger
    def pretty(self):# -> Ripgrepy
        """
        This is a convenience alias for --color always --heading
        --line-number. This flag is useful when you still want pretty
        output even if you're piping ripgrep to another program or file.
        For example: rg -p foo | less -R.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--pretty")
        return self

    @_logger
    def quite(self):# -> Ripgrepy
        """
        Do not print anything to stdout. If a match is found in a file,
        then ripgrep will stop searching. This is useful when ripgrep is
        used only for its exit code (which will be an error if no matches
        are found).

        When --files is used, then ripgrep will stop finding files after
        finding the first file that matches all ignore rules.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--quite")
        return self

    @_logger
    def regex_size_limit(self, num_suffix: str):# -> Ripgrepy
        """
        The upper size limit of the compiled regex. The default limit is
        10M.

        The argument accepts the same size suffixes as allowed in the
        --max-filesize flag.

        :param num_suffix: Number + suffix
        :type num_suffix: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--regex-size-limit "{num_suffix}"')
        return self

    @_logger
    def regexp(self, pattern: str):# -> Ripgrepy
        """
        A pattern to search for. This option can be provided multiple
        times, where all patterns given are searched. Lines matching at
        least one of the provided patterns are printed. This flag can also
        be used when searching for patterns that start with a dash.

        For example, to search for the literal -foo, you can use this flag:

            rg -e -foo

        You can also use the special -- delimiter to indicate that no more
        flags will be provided. Namely, the following is equivalent to the
        above:

            rg -- -foo

        :param pattern: Regex pattern
        :type pattern: str
        :return: self
        :rtype: Ripgrepy
        """
        self.regex_pattern = ""
        self.command.append(f'--regexp "{pattern}"')
        return self

    @_logger
    def replace(self, replacement_text: str):# -> Ripgrepy
        """
        Replace every match with the text given when printing results.
        Neither this flag nor any other ripgrep flag will modify your
        files.

        Capture group indices (e.g., $5) and names (e.g., $foo) are
        supported in the replacement string.

        Note that the replacement by default replaces each match, and NOT
        the entire line. To replace the entire line, you should match the
        entire line.

        This flag can be used with the -o/--only-matching flag.

        :param replacement_text: Replacement text. Groups are supported
        :type replacement_text: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f"--replace '{replacement_text}'")
        return self

    @_logger
    def search_zip(self):# -> Ripgrepy
        """
        Search in compressed files. Currently gzip, bzip2, xz, LZ4, LZMA,
        Brotli and Zstd files are supported. This option expects the
        decompression binaries to be available in your PATH.

        This flag can be disabled with --no-search-zip.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--search-zip")
        return self

    @_logger
    def smart_case(self) :# -> Ripgrepy
        """
        Searches case insensitively if the pattern is all lowercase. Search
        case sensitively otherwise.

        This overrides the -s/--case-sensitive and -i/--ignore-case flags.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--smart-case")
        return self

    @_logger
    def sort(self, sort_by: str):# -> Ripgrepy
        """
        This flag enables sorting of results in ascending order. The
        possible values for this flag are:

            path        Sort by file path.
            modified    Sort by the last modified time on a file.
            accessed    Sort by the last accessed time on a file.
            created     Sort by the creation time on a file.
            none        Do not sort results.

        If the sorting criteria isn't available on your system (for
        example, creation time is not available on ext4 file systems), then
        ripgrep will attempt to detect this and print an error without
        searching any results. Otherwise, the sort order is unspecified.

        To sort results in reverse or descending order, use the --sortr
        flag. Also, this flag overrides --sortr.

        Note that sorting results currently always forces ripgrep to
        abandon parallelism and run in a single thread.

        :param sort_by: Sort by path, modified, accessed, created, none
        :type sort_by: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--sort "{sort_by}"')
        return self

    @_logger
    def sortr(self, sort_by: str):# -> Ripgrepy
        """
        This flag enables sorting of results in descending order. The
        possible values for this flag are:

            path        Sort by file path.
            modified    Sort by the last modified time on a file.
            accessed    Sort by the last accessed time on a file.
            created     Sort by the creation time on a file.
            none        Do not sort results.

        If the sorting criteria isn't available on your system (for
        example, creation time is not available on ext4 file systems), then
        ripgrep will attempt to detect this and print an error without
        searching any results. Otherwise, the sort order is unspecified.

        To sort results in ascending order, use the --sort flag. Also, this
        flag overrides --sort.

        Note that sorting results currently always forces ripgrep to
        abandon parallelism and run in a single thread.

        :param sort_by: Sort desending by. Valid options are accessed, modified, path, created, none
        :type sort_by: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--sortr "{sort_by}"')
        return self

    @_logger
    def stats(self):# -> Ripgrepy
        """
        Print aggregate statistics about this ripgrep search. When this
        flag is present, ripgrep will print the following stats to stdout
        at the end of the search: number of matched lines, number of files
        with matches, number of files searched, and the time taken for the
        entire search to complete.

        This set of aggregate statistics may expand over time.

        Note that this flag has no effect if --files, --files-with-matches
        or --files-without-match is passed.

        This flag can be disabled with --no-stats.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--stats")
        return self

    @_logger
    def text(self):# -> Ripgrepy
        """
        Search binary files as if they were text. When this flag is
        present, ripgrep's binary file detection is disabled. This means
        that when a binary file is searched, its contents may be printed if
        there is a match. This may cause escape codes to be printed that
        alter the behavior of your terminal.

        When binary file detection is enabled it is imperfect. In general,
        it uses a simple heuristic. If a NUL byte is seen during search,
        then the file is considered binary and search stops (unless this
        flag is present). Alternatively, if the --binary flag is used, then
        ripgrep will only quit when it sees a NUL byte after it sees a
        match (or searches the entire file).

        This flag can be disabled with --no-text. It overrides the --binary
        flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--text")
        return self

    @_logger
    def threads(self, num: int):# -> Ripgrepy
        """
        The approximate number of threads to use. A value of 0 (which is
        the default) causes ripgrep to choose the thread count using
        heuristics.

        :param num: Number of threads
        :type num: int
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f"--threads {num}")
        return self

    @_logger
    def trim(self):# -> Ripgrepy
        """
        When set, all ASCII whitespace at the beginning of each line
        printed will be trimmed.

        This flag can be disabled with --no-trim.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--trim")
        return self

    @_logger
    def type_(self, type_pattern: str):# -> Ripgrepy
        """
        Only search files matching TYPE. Multiple type flags may be
        provided. Use the --type-list flag to list all available types.

        :param type_pattern: Type pattern
        :type type_pattern: str
        :return: [description]
        :rtype: Ripgrepy
        """
        self.command.append(f'--type "{type_pattern}"')
        return self

    @_logger
    def type_add(self, type_spec: str):# -> Ripgrepy
        """
        Add a new glob for a particular file type. Only one glob can be
        added at a time. Multiple --type-add flags can be provided. Unless
        --type-clear is used, globs are added to any existing globs defined
        inside of ripgrep.

        Note that this MUST be passed to every invocation of ripgrep. Type
        settings are NOT persisted.

        Example:

        >>>    rg --type-add 'foo:*.foo' -tfoo PATTERN.

        --type-add can also be used to include rules from other types with
        the special include directive. The include directive permits
        specifying one or more other type names (separated by a comma) that
        have been defined and its rules will automatically be imported into
        the type specified. For example, to create a type called src that
        matches C++, Python and Markdown files, one can use:

        >>>    --type-add 'src:include:cpp,py,md'

        Additional glob rules can still be added to the src type by using
        the --type-add flag again:

        >>>    --type-add 'src:include:cpp,py,md' --type-add 'src:*.foo'

        Note that type names must consist only of Unicode letters or
        numbers. Punctuation characters are not allowed.

        :param type_spec: Type spec
        :type type_spec: str
        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f'--type-add "{type_spec}"')
        return self

    @_logger
    def type_clear(self):# -> Ripgrepy
        """
        Clear the file type globs previously defined for TYPE. This only
        clears the default type definitions that are found inside of
        ripgrep.

        Note that this MUST be passed to every invocation of ripgrep. Type
        settings are NOT persisted.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--type-clear")
        return self

    @_logger
    def type_list(self):# -> Ripgrepy
        """
        Show all supported file types and their corresponding globs.

        :return: self
        :rtype: Ripgrepy
        """
        self.regex_pattern = ""
        self.path = ""
        self.command.append("--type-list")
        return self

    @_logger
    def type_not(self, type_pattern: str):# -> Ripgrepy
        """
        Do not search files matching TYPE. Multiple type-not flags may be
        provided. Use the --type-list flag to list all available types

        :param type_pattern: Type pattern
        :type type_pattern: str
        :return: [description]
        :rtype: Ripgrepy
        """
        self.command.append(f'--type-not "{type_pattern}"')
        return self

    @_logger
    def unrestricted(self):# -> Ripgrepy
        """
        Reduce the level of "smart" searching. A single -u won't respect
        .gitignore (etc.) files. Two -u flags will additionally search
        hidden files and directories. Three -u flags will additionally
        search binary files.

        rg -uuu is roughly equivalent to grep -r.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--unrestricted")
        return self

    @_logger
    def vimgrep(self):# -> Ripgrepy
        """
        Show results with every match on its own line, including line
        numbers and column numbers. With this option, a line with more than
        one match will be printed more than once.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--vimgrep")
        return self

    @_logger
    def with_filename(self):# -> Ripgrepy
        """
        Display the file path for matches. This is the default when more
        than one file is searched. If --heading is enabled (the default
        when printing to a terminal), the file path will be shown above
        clusters of matches from each file; otherwise, the file name will
        be shown as a prefix for each matched line.

        This flag overrides --no-filename.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--with-filename")
        return self

    @_logger
    def word_regexp(self):# -> Ripgrepy
        """
        Only show matches surrounded by word boundaries. This is roughly
        equivalent to putting \\b before and after all of the search
        patterns.

        This overrides the --line-regexp flag.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--word-regexp")
        return self

    ### Options for version 12 of ripgrep
    @_logger
    def no_unicode(self):# -> Ripgrepy
        """
        By default, ripgrep will enable "Unicode mode" in all of its
        regexes. This has a number of consequences:

        ·   . will only match valid UTF-8 encoded scalar values.

        ·   Classes like \w, \s, \d are all Unicode aware and much bigger
            than their ASCII only versions.

        ·   Case insensitive matching will use Unicode case folding.

        ·   A large array of classes like \p{Emoji} are available.

        ·   Word boundaries (\\b and \B) use the Unicode definition of a
            word character.

            In some cases it can be desirable to turn these things off. The
            --no-unicode flag will do exactly that.

            For PCRE2 specifically, Unicode mode represents a critical
            trade off in the user experience of ripgrep. In particular,
            unlike the default regex engine, PCRE2 does not support the
            ability to search possibly invalid UTF-8 with Unicode features
            enabled. Instead, PCRE2 requires that everything it searches
            when Unicode mode is enabled is valid UTF-8. (Or valid
            UTF-16/UTF-32, but for the purposes of ripgrep, we only discuss
            UTF-8.) This means that if you have PCRE2's Unicode mode
            enabled and you attempt to search invalid UTF-8, then the
            search for that file will halt and print an error. For this
            reason, when PCRE2's Unicode mode is enabled, ripgrep will
            automatically "fix" invalid UTF-8 sequences by replacing them
            with the Unicode replacement codepoint. This penalty does not
            occur when using the default regex engine.

            If you would rather see the encoding errors surfaced by PCRE2
            when Unicode mode is enabled, then pass the --no-encoding flag
            to disable all transcoding.

            The --no-unicode flag can be disabled with --unicode. Note that
            --no-pcre2-unicode and --pcre2-unicode are aliases for
            --no-unicode and --unicode, respectively.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append("--no-unicode")
        return self

    @_logger
    def engine(self, engine: str):# -> Ripgrepy
        """
        Specify which regular expression engine to use. When you choose a
        regex engine, it applies that choice for every regex provided to
        ripgrep (e.g., via multiple -e/--regexp or -f/--file flags).

        Accepted values are default, pcre2, or auto.

        The default value is default, which is the fastest and should be
        good for most use cases. The pcre2 engine is generally useful when
        you want to use features such as look-around or backreferences.
        auto will dynamically choose between supported regex engines
        depending on the features used in a pattern on a best effort basis.

        Note that the pcre2 engine is an optional ripgrep feature. If PCRE2
        wasn't including in your build of ripgrep, then using this flag
        will result in ripgrep printing an error message and exiting.

        This overrides previous uses of --pcre2 and --auto-hybrid-regex
        flags.

        :return: self
        :rtype: Ripgrepy
        """
        self.command.append(f"--engine {engine}")
        return self



import json
import click
from cores import stdvarinit
from cores import fs
import requests
import os
from cores.stdpackage import PlatformPackageManager
@stdvarinit.cli.group("platform", short_help="platform manager")
def cli():
    pass


@cli.command("search", short_help="Search for development platform")
@click.argument("query", required=False)
@click.option("--json-output", is_flag=True)
def platform_search(query, json_output):
    load_data = []
    stdplatforms_json = stdvarinit.session_dir + "/stdplatforms.json"
    url = "https://stduino-generic.pkg.coding.net/stduino_packages/stdplatforms/stdplatforms.json?version=latest"
    if os.path.isfile(stdplatforms_json):
        if fs.is_connected():
            html = requests.head(url)  # 用head方法去请求资源头
            if html.status_code == 200:
                online_size = int(html.headers['Content-Length'])  # 提取出来的是个数字str
                current_size = os.path.getsize(stdplatforms_json)
                if online_size==current_size:
                    load_data=fs.load_json(stdplatforms_json)
                    #click.echo()
                else:
                    fs.delete_file(stdplatforms_json)
                    fs.fast_download(url,stdplatforms_json)
                    load_data=fs.load_json(stdplatforms_json)
        else:
            load_data=fs.load_json(stdplatforms_json)

    else:
        if fs.is_connected():
            fs.fast_download(url, stdplatforms_json)
            load_data=fs.load_json(stdplatforms_json)
    if json_output:
        click.echo(json.dumps(load_data))
    else:
        _print_platforms(load_data)
    #json.dumps(pio_boards)

@cli.command("install", short_help="Install new development platform")
@click.option("--platform", '-p', help='target install platform.')
@click.option("-s", "--silent", is_flag=True, help="Suppress progress reporting")
@click.option(
    "-f",
    "--force",
    is_flag=True,
    help="Reinstall/redownload dev/platform and its packages if exist",
)
def platform_install(  # pylint: disable=too-many-arguments
    platform,
    silent,
    force,
):
    return _platform_install(
        platform,
        silent,
        force,
    )


def _platform_install(  # pylint: disable=too-many-arguments
    platforms,
    silent=False,
    force=False,
):
    print(platforms)
    pm = PlatformPackageManager()
    pkg = pm.install(
        spec=platforms,
        silent=silent,
        force=force,
    )
    if pkg and not silent:
        click.secho(
            "The platform '%s' has been successfully installed!\n"
            "The rest of the packages will be installed later "
            "depending on your build environment." % platforms,
            fg="green",
        )


@cli.command("install2", short_help="Install new development platform")
@click.argument("platforms", nargs=-1, required=True, metavar="[PLATFORM...]")
@click.option("--with-package", multiple=True)
@click.option("--without-package", multiple=True)
@click.option("--skip-default-package", is_flag=True)
@click.option("--with-all-packages", is_flag=True)
@click.option("-s", "--silent", is_flag=True, help="Suppress progress reporting")
@click.option(
    "-f",
    "--force",
    is_flag=True,
    help="Reinstall/redownload dev/platform and its packages if exist",
)
def platforms_install(  # pylint: disable=too-many-arguments
    platforms,
    with_package,
    without_package,
    skip_default_package,
    with_all_packages,
    silent,
    force,
):
    return _platform_install(
        platforms,
        with_package,
        without_package,
        skip_default_package,
        with_all_packages,
        silent,
        force,
    )


def _platforms_install(  # pylint: disable=too-many-arguments
    platforms,
    with_package=None,
    without_package=None,
    skip_default_package=False,
    with_all_packages=False,
    silent=False,
    force=False,
):
    #pm = PlatformPackageManager()
    for platform in platforms:
        pkg = pm.install(
            spec=platform,
            with_packages=with_package or [],
            without_packages=without_package or [],
            skip_default_package=skip_default_package,
            with_all_packages=with_all_packages,
            silent=silent,
            force=force,
        )
        if pkg and not silent:
            click.secho(
                "The platform '%s' has been successfully installed!\n"
                "The rest of the packages will be installed later "
                "depending on your build environment." % platform,
                fg="green",
            )

@cli.command("list", short_help="List all installed platforms")
@click.option("--json-output", is_flag=True)
def platform_list(json_output):
    platforms = []
    # pm = PlatformPackageManager()
    # for pkg in pm.get_installed():
    #     platforms.append(
    #         _get_installed_platform_data(pkg, with_boards=False, expose_packages=False)
    #     )
    #
    # platforms = sorted(platforms, key=lambda manifest: manifest["name"])
    if json_output:
        click.echo(json.dumps(platforms))
    else:
        print(platforms)

def _install_platform(board):
    pass

#self.pio_env + " project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework

def _print_platforms(platforms):
    for platform in platforms:
        click.echo(
            "{name} ~ {title}".format(
                name=click.style(platform["name"], fg="cyan"), title=platform["title"]
            )
        )
        click.echo("=" * (3 + len(platform["name"] + platform["title"])))
        click.echo(platform["description"])
        click.echo()
        if "homepage" in platform:
            click.echo("Home: %s" % platform["homepage"])
        if "frameworks" in platform and platform["frameworks"]:
            click.echo("Frameworks: %s" % ", ".join(platform["frameworks"]))
        if "packages" in platform:
            click.echo("Packages: %s" % ", ".join(platform["packages"]))
        if "version" in platform:
            if "__src_url" in platform:
                click.echo(
                    "Version: %s (%s)" % (platform["version"], platform["__src_url"])
                )
            else:
                click.echo("Version: " + platform["version"])
        click.echo()
    pass
if __name__ == '__main__':
    stdvarinit.cli()




# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
from function.conf import res

from function.language import zh_hans as reso
try:

    if res.msg == "1":  # 中
        from ..language import zh_hans as reso

        pass
    elif res.msg == "2":  # 英
        from ..language import en as reso

        pass
    elif res.msg == "python3":  # 俄
        from ..language import russian as reso

        pass
    elif res.msg == "4":  # 德
        from ..language import german as reso

        pass
    elif res.msg == "5":  # 日
        from ..language import japanese as reso

        pass
    elif res.msg == "6":  # 韩
        from ..language import korean as reso

        pass
    elif res.msg == "7":  # 西班牙
        from ..language import spanish as reso

        pass
    elif res.msg == "8":  # 法
        from ..language import french as reso

        pass
    elif res.msg == "9":  # 阿拉伯语
        from ..language import arabic as reso

        pass
    else:  # 英
        from ..language import en as reso

        pass

except:
    print("language")



from cores import stdvarinit
import json
from cores import fs
import requests
import os
import click
@stdvarinit.cli.group("boards", short_help="boards manager")
def cli():
    pass
@cli.command("installed", short_help="find board")
def boards_installed():
    pass



@cli.command("search", short_help="Search for development board")
@click.argument("query", required=False)
@click.option("--json-output", is_flag=True)
def board_search(query, json_output):
    load_data = []
    stdboards_json = stdvarinit.session_dir + "/stdboards.json"
    url = "https://stduino-generic.pkg.coding.net/stduino_packages/stdplatforms/stdboards.json?version=latest"
    if os.path.isfile(stdboards_json):
        if fs.is_connected():
            html = requests.head(url)  # 用head方法去请求资源头
            if html.status_code == 200:
                online_size = int(html.headers['Content-Length'])  # 提取出来的是个数字str
                current_size = os.path.getsize(stdboards_json)
                if online_size==current_size:
                    load_data=fs.load_json(stdboards_json)
                    #click.echo()
                else:
                    fs.delete_file(stdboards_json)
                    fs.fast_download(url,stdboards_json)
                    load_data=fs.load_json(stdboards_json)
        else:
            load_data=fs.load_json(stdboards_json)

    else:
        if fs.is_connected():
            fs.fast_download(url, stdboards_json)
            load_data=fs.load_json(stdboards_json)
    if json_output:
        click.echo(json.dumps(load_data))
    else:
        click.echo(json.dumps(load_data))
        pass
        #_#print_platforms(load_data)
    #json.dumps(pio_boards)
def test():
    target_path="C:/Users/debug/.stduino/platforms/dS1"
    if not os.path.isdir(target_path):
        print(0)
    else:
        from multiprocessing import cpu_count
        print(cpu_count())


# 大小写区分待解决 win下
if __name__ == '__main__':
    test()


from distutils.core import setup
from setuptools import find_packages
# python setup.py bdist_wheel # 打包为whl文件
# python setup.py sdist # 打包为tar.gz文件
#twine upload dist/*
#qq

setup(name='stduino',  # 包名
      version='0.1.10',  # 版本号
      description='Stduino IDE For win、mac、linux',
      long_description='Stduino IDE For Embedded development.',
      author='文涛',
      author_email='33672008@qq.com',
      url='http://www.stduino.com',
      license='',#'pyqtwebengine>=5.15',
      install_requires=['click==7.1.2','pyserial>=2.0','pyqt5==5.13.0','pywebview>=3.0','requests>=2.20','qdarkstyle==2.8.1','pygdbmi>=0.10','qscintilla==2.11.2','watchdog>=2.1'],
      classifiers=[
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Natural Language :: Chinese (Simplified)',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Topic :: Utilities'
      ],
      keywords='Stduino IDE',
      packages=find_packages('src'),  # 必填，就是包的代码主目录
      package_dir={'': 'src'},  # 必填
      include_package_data=True,
      )



# -*- coding: utf-8 -*-

"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
from .stdedit import stdinit
#from function.cores.stdedit import stdinit

from PyQt5.Qsci import QsciScintilla
#from PyQt5.QtWidgets import QMessageBox
class Commenter():
    def __init__(self, sci, comment_str):
        self.sci = sci
        self.comment_str = comment_str
        self.sel_regions = []


    def toggle_comments(self):
        try:
            lines = self.selected_lines()
            if len(lines) <= 0:
                return
            all_commented = True
            for line in lines:
                if not self.sci.text(line).strip().startswith(self.comment_str):
                    all_commented = False
            if not all_commented:
                self.comment_lines(lines)
            else:
                self.uncomment_lines(lines)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def selections(self):
        try:
            regions = []
            for i in range(self.sci.SendScintilla(QsciScintilla.SCI_GETSELECTIONS)):
                regions.append({
                    'begin': self.sci.SendScintilla(QsciScintilla.SCI_GETSELECTIONNSTART, i),
                    'end': self.sci.SendScintilla(QsciScintilla.SCI_GETSELECTIONNEND, i)
                })

            return regions
        except:
            stdinit.std_signal_gobal.stdprintln()


    def selected_lines(self):
        try:
            self.sel_regions = []
            all_lines = []
            regions = self.selections()
            for r in regions:
                start_line = self.sci.SendScintilla(QsciScintilla.SCI_LINEFROMPOSITION, r['begin'])
                end_line = self.sci.SendScintilla(QsciScintilla.SCI_LINEFROMPOSITION, r['end'])
                for cur_line in range(start_line, end_line + 1):
                    if not cur_line in all_lines:
                        all_lines.append(cur_line)
                if r['begin'] <= r['end']:
                    self.sel_regions.append(r)
            return all_lines
        except:
            stdinit.std_signal_gobal.stdprintln()


    def comment_lines(self, lines):
        try:
            indent = self.sci.indentation(lines[0])
            for line in lines:
                indent = min(indent, self.sci.indentation(line))
            self.sci.beginUndoAction()
            for line in lines:
                self.adjust_selections(line, indent)
                self.sci.insertAt(self.comment_str, line, indent)
            self.sci.endUndoAction()
            self.restore_selections()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def uncomment_lines(self, lines):
        try:
            self.sci.beginUndoAction()
            for line in lines:
                line_start = self.sci.SendScintilla(QsciScintilla.SCI_POSITIONFROMLINE, line)
                line_end = self.sci.SendScintilla(QsciScintilla.SCI_GETLINEENDPOSITION, line)
                if line_start == line_end:
                    continue
                if line_end - line_start < len(self.comment_str):
                    continue
                done = False
                for c in range(line_start, line_end - len(self.comment_str) + 1):
                    source_str = self.sci.text(c, c + len(self.comment_str))
                    if (source_str == self.comment_str):
                        self.sci.SendScintilla(QsciScintilla.SCI_DELETERANGE, c, len(self.comment_str))
                        break
            self.sci.endUndoAction()
        except:
            stdinit.std_signal_gobal.stdprintln()

    def restore_selections(self):
        try:
            if (len(self.sel_regions) > 0):
                first = True
                for r in self.sel_regions:
                    if first:
                        self.sci.SendScintilla(QsciScintilla.SCI_SETSELECTION, r['begin'], r['end'])
                        first = False
                    else:
                        self.sci.SendScintilla(QsciScintilla.SCI_ADDSELECTION, r['begin'], r['end'])
        except:
            stdinit.std_signal_gobal.stdprintln()


    def adjust_selections(self, line, indent):
        try:
            for r in self.sel_regions:
                if self.sci.positionFromLineIndex(line, indent) <= r['begin']:
                    r['begin'] += len(self.comment_str)
                    r['end'] += len(self.comment_str)
                elif self.sci.positionFromLineIndex(line, indent) < r['end']:
                    r['end'] += len(self.comment_str)
        except:
            stdinit.std_signal_gobal.stdprintln()






# Copyright (c) 2014-present PlatformIO <contact@platformio.org>
# Copyright 2020 MongoDB Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# pylint: disable=unused-argument, protected-access, unused-variable, import-error
# Original: https://github.com/mongodb/mongo/blob/master/site_scons/site_tools/compilation_db.py

from __future__ import absolute_import

import itertools
import json
import os

import SCons

from platformio.builder.tools.platformio import SRC_ASM_EXT, SRC_C_EXT, SRC_CXX_EXT
from platformio.proc import where_is_program

# Implements the ability for SCons to emit a compilation database for the MongoDB project. See
# http://clang.llvm.org/docs/JSONCompilationDatabase.html for details on what a compilation
# database is, and why you might want one. The only user visible entry point here is
# 'env.CompilationDatabase'. This method takes an optional 'target' to name the file that
# should hold the compilation database, otherwise, the file defaults to compile_commands.json,
# which is the name that most clang tools search for by default.

# Is there a better way to do this than this global? Right now this exists so that the
# emitter we add can record all of the things it emits, so that the scanner for the top level
# compilation database can access the complete list, and also so that the writer has easy
# access to write all of the files. But it seems clunky. How can the emitter and the scanner
# communicate more gracefully?
__COMPILATION_DB_ENTRIES = []


# We make no effort to avoid rebuilding the entries. Someday, perhaps we could and even
# integrate with the cache, but there doesn't seem to be much call for it.
class __CompilationDbNode(SCons.Node.Python.Value):
    def __init__(self, value):
        SCons.Node.Python.Value.__init__(self, value)
        self.Decider(changed_since_last_build_node)


def changed_since_last_build_node(*args, **kwargs):
    """Dummy decider to force always building"""
    return True


def makeEmitCompilationDbEntry(comstr):
    """
    Effectively this creates a lambda function to capture:
    * command line
    * source
    * target
    :param comstr: unevaluated command line
    :return: an emitter which has captured the above
    """
    user_action = SCons.Action.Action(comstr)

    def EmitCompilationDbEntry(target, source, env):
        """
        This emitter will be added to each c/c++ object build to capture the info needed
        for clang tools
        :param target: target node(s)
        :param source: source node(s)
        :param env: Environment for use building this node
        :return: target(s), source(s)
        """

        # Resolve absolute path of toolchain
        for cmd in ("CC", "CXX", "AS"):
            if cmd not in env:
                continue
            if os.path.isabs(env[cmd]):
                continue
            env[cmd] = where_is_program(
                env.subst("$%s" % cmd), env.subst("${ENV['PATH']}")
            )

        dbtarget = __CompilationDbNode(source)

        entry = env.__COMPILATIONDB_Entry(
            target=dbtarget,
            source=[],
            __COMPILATIONDB_UTARGET=target,
            __COMPILATIONDB_USOURCE=source,
            __COMPILATIONDB_UACTION=user_action,
            __COMPILATIONDB_ENV=env,
        )

        # Technically, these next two lines should not be required: it should be fine to
        # cache the entries. However, they don't seem to update properly. Since they are quick
        # to re-generate disable caching and sidestep this problem.
        env.AlwaysBuild(entry)
        env.NoCache(entry)

        __COMPILATION_DB_ENTRIES.append(dbtarget)

        return target, source

    return EmitCompilationDbEntry


def CompilationDbEntryAction(target, source, env, **kw):
    """
    Create a dictionary with evaluated command line, target, source
    and store that info as an attribute on the target
    (Which has been stored in __COMPILATION_DB_ENTRIES array
    :param target: target node(s)
    :param source: source node(s)
    :param env: Environment for use building this node
    :param kw:
    :return: None
    """

    command = env["__COMPILATIONDB_UACTION"].strfunction(
        target=env["__COMPILATIONDB_UTARGET"],
        source=env["__COMPILATIONDB_USOURCE"],
        env=env["__COMPILATIONDB_ENV"],
    )

    entry = {
        "directory": env.Dir("#").abspath,
        "command": command,
        "file": str(env["__COMPILATIONDB_USOURCE"][0]),
    }

    target[0].write(entry)


def WriteCompilationDb(target, source, env):
    entries = []

    for s in __COMPILATION_DB_ENTRIES:
        item = s.read()
        item["file"] = os.path.abspath(item["file"])
        entries.append(item)

    with open(str(target[0]), mode="w", encoding="utf8") as target_file:
        json.dump(
            entries, target_file, sort_keys=True, indent=4, separators=(",", ": ")
        )


def ScanCompilationDb(node, env, path):
    return __COMPILATION_DB_ENTRIES


def generate(env, **kwargs):
    static_obj, shared_obj = SCons.Tool.createObjBuilders(env)

    env["COMPILATIONDB_COMSTR"] = kwargs.get(
        "COMPILATIONDB_COMSTR", "Building compilation database $TARGET"
    )

    components_by_suffix = itertools.chain(
        itertools.product(
            [".%s" % ext for ext in SRC_C_EXT],
            [
                (static_obj, SCons.Defaults.StaticObjectEmitter, "$CCCOM"),
                (shared_obj, SCons.Defaults.SharedObjectEmitter, "$SHCCCOM"),
            ],
        ),
        itertools.product(
            [".%s" % ext for ext in SRC_CXX_EXT],
            [
                (static_obj, SCons.Defaults.StaticObjectEmitter, "$CXXCOM"),
                (shared_obj, SCons.Defaults.SharedObjectEmitter, "$SHCXXCOM"),
            ],
        ),
        itertools.product(
            [".%s" % ext for ext in SRC_ASM_EXT],
            [(static_obj, SCons.Defaults.StaticObjectEmitter, "$ASCOM")],
        ),
    )

    for entry in components_by_suffix:
        suffix = entry[0]
        builder, base_emitter, command = entry[1]

        # Assumes a dictionary emitter
        emitter = builder.emitter[suffix]
        builder.emitter[suffix] = SCons.Builder.ListEmitter(
            [emitter, makeEmitCompilationDbEntry(command)]
        )

    env["BUILDERS"]["__COMPILATIONDB_Entry"] = SCons.Builder.Builder(
        action=SCons.Action.Action(CompilationDbEntryAction, None),
    )

    env["BUILDERS"]["__COMPILATIONDB_Database"] = SCons.Builder.Builder(
        action=SCons.Action.Action(WriteCompilationDb, "$COMPILATIONDB_COMSTR"),
        target_scanner=SCons.Scanner.Scanner(
            function=ScanCompilationDb, node_class=None
        ),
    )

    def CompilationDatabase(env, target):
        result = env.__COMPILATIONDB_Database(target=target, source=[])

        env.AlwaysBuild(result)
        env.NoCache(result)

        return result

    env.AddMethod(CompilationDatabase, "CompilationDatabase")


def exists(env):
    return True
